@@ -24,11 +24,11 @@ class AMLogSubscriberTest < ActionMailer::TestCase
   end
 
   def test_deliver_is_notified
-    BaseMailer.welcome.deliver_now
+    BaseMailer.welcome(message_id: "123@abc").deliver_now
     wait
 
     assert_equal(1, @logger.logged(:info).size)
-    assert_match(/Sent mail to system@test\.lindsaar\.net/, @logger.logged(:info).first)
+    assert_match(/Delivered mail 123@abc/, @logger.logged(:info).first)
 
     assert_equal(2, @logger.logged(:debug).size)
     assert_match(/BaseMailer#welcome: processed outbound mail in [\d.]+ms/, @logger.logged(:debug).first)
@@ -37,9 +37,25 @@ class AMLogSubscriberTest < ActionMailer::TestCase
     BaseMailer.deliveries.clear
   end
 
+  def test_deliver_message_when_perform_deliveries_is_false
+    BaseMailer.welcome_without_deliveries(message_id: "123@abc").deliver_now
+    wait
+
+    assert_equal(1, @logger.logged(:info).size)
+    assert_match("Skipped delivery of mail 123@abc as `perform_deliveries` is false", @logger.logged(:info).first)
+
+    assert_equal(2, @logger.logged(:debug).size)
+    assert_match(/BaseMailer#welcome_without_deliveries: processed outbound mail in [\d.]+ms/, @logger.logged(:debug).first)
+    assert_match("Welcome", @logger.logged(:debug).second)
+  ensure
+    BaseMailer.deliveries.clear
+  end
+
   def test_receive_is_notified
     fixture = File.read(File.expand_path("fixtures/raw_email", __dir__))
-    TestMailer.receive(fixture)
+    assert_deprecated do
+      TestMailer.receive(fixture)
+    end
     wait
     assert_equal(1, @logger.logged(:info).size)
     assert_match(/Received mail/, @logger.logged(:info).first)
