@@ -21,6 +21,11 @@ class BaseMailer < ActionMailer::Base
     mail(template_name: "welcome", template_path: path)
   end
 
+  def welcome_without_deliveries(hash = {})
+    mail({ template_name: "welcome" }.merge!(hash))
+    mail.perform_deliveries = false
+  end
+
   def html_only(hash = {})
     mail(hash)
   end
@@ -57,6 +62,10 @@ class BaseMailer < ActionMailer::Base
     mail(hash)
   end
 
+  def implicit_multipart_formats(hash = {})
+    mail(hash)
+  end
+
   def implicit_with_locale(hash = {})
     mail(hash)
   end
@@ -106,6 +115,13 @@ class BaseMailer < ActionMailer::Base
     mail(template_name: template_name)
   end
 
+  def implicit_different_template_with_block(template_name = "")
+    mail(template_name: template_name) do |format|
+      format.text
+      format.html
+    end
+  end
+
   def explicit_different_template(template_name = "")
     mail do |format|
       format.text { render template: "#{mailer_name}/#{template_name}" }
