@@ -1,11 +1,12 @@
+# frozen_string_literal: true
+
 module ActionDispatch
   module Http
     module FilterRedirect
+      FILTERED = "[FILTERED]" # :nodoc:
 
-      FILTERED = '[FILTERED]'.freeze # :nodoc:
-
-      def filtered_location
-        if !location_filter.empty? && location_filter_match?
+      def filtered_location # :nodoc:
+        if location_filter_match?
           FILTERED
         else
           location
@@ -13,25 +14,23 @@ module ActionDispatch
       end
 
     private
-
-      def location_filter
-        if request.present?
-          request.env['action_dispatch.redirect_filter'] || []
+      def location_filters
+        if request
+          request.get_header("action_dispatch.redirect_filter") || []
         else
           []
         end
       end
 
       def location_filter_match?
-        location_filter.any? do |filter|
+        location_filters.any? do |filter|
           if String === filter
             location.include?(filter)
           elsif Regexp === filter
-            location.match(filter)
+            location =~ filter
           end
         end
       end
-
     end
   end
 end
