@@ -26,11 +26,20 @@ class RenderJSTest < ActionController::TestCase
   def test_render_vanilla_js
     get :render_vanilla_js_hello, xhr: true
     assert_equal "alert('hello')", @response.body
-    assert_equal "text/javascript", @response.content_type
+    assert_equal "text/javascript", @response.media_type
   end
 
   def test_should_render_js_partial
     get :show_partial, format: "js", xhr: true
     assert_equal "partial js", @response.body
   end
+
+  def test_should_not_trigger_content_type_deprecation
+    original = ActionDispatch::Response.return_only_media_type_on_content_type
+    ActionDispatch::Response.return_only_media_type_on_content_type = true
+
+    assert_not_deprecated { get :render_vanilla_js_hello, xhr: true }
+  ensure
+    ActionDispatch::Response.return_only_media_type_on_content_type = original
+  end
 end
