@@ -80,7 +80,7 @@ class RenderJsonTest < ActionController::TestCase
   def test_render_json_nil
     get :render_json_nil
     assert_equal "null", @response.body
-    assert_equal "application/json", @response.content_type
+    assert_equal "application/json", @response.media_type
   end
 
   def test_render_json_render_to_string
@@ -91,7 +91,7 @@ class RenderJsonTest < ActionController::TestCase
   def test_render_json
     get :render_json_hello_world
     assert_equal '{"hello":"world"}', @response.body
-    assert_equal "application/json", @response.content_type
+    assert_equal "application/json", @response.media_type
   end
 
   def test_render_json_with_status
@@ -103,35 +103,53 @@ class RenderJsonTest < ActionController::TestCase
   def test_render_json_with_callback
     get :render_json_hello_world_with_callback, xhr: true
     assert_equal '/**/alert({"hello":"world"})', @response.body
-    assert_equal "text/javascript", @response.content_type
+    assert_equal "text/javascript", @response.media_type
   end
 
   def test_render_json_with_custom_content_type
     get :render_json_with_custom_content_type, xhr: true
     assert_equal '{"hello":"world"}', @response.body
-    assert_equal "text/javascript", @response.content_type
+    assert_equal "text/javascript", @response.media_type
   end
 
   def test_render_symbol_json
     get :render_symbol_json
     assert_equal '{"hello":"world"}', @response.body
-    assert_equal "application/json", @response.content_type
+    assert_equal "application/json", @response.media_type
   end
 
   def test_render_json_with_render_to_string
     get :render_json_with_render_to_string
     assert_equal '{"hello":"partial html"}', @response.body
-    assert_equal "application/json", @response.content_type
+    assert_equal "application/json", @response.media_type
   end
 
   def test_render_json_forwards_extra_options
     get :render_json_with_extra_options
     assert_equal '{"a":"b"}', @response.body
-    assert_equal "application/json", @response.content_type
+    assert_equal "application/json", @response.media_type
   end
 
   def test_render_json_calls_to_json_from_object
     get :render_json_without_options
     assert_equal '{"a":"b"}', @response.body
   end
+
+  def test_should_not_trigger_content_type_deprecation
+    original = ActionDispatch::Response.return_only_media_type_on_content_type
+    ActionDispatch::Response.return_only_media_type_on_content_type = true
+
+    assert_not_deprecated { get :render_json_hello_world }
+  ensure
+    ActionDispatch::Response.return_only_media_type_on_content_type = original
+  end
+
+  def test_should_not_trigger_content_type_deprecation_with_callback
+    original = ActionDispatch::Response.return_only_media_type_on_content_type
+    ActionDispatch::Response.return_only_media_type_on_content_type = true
+
+    assert_not_deprecated { get :render_json_hello_world_with_callback, xhr: true }
+  ensure
+    ActionDispatch::Response.return_only_media_type_on_content_type = original
+  end
 end
