@@ -62,10 +62,14 @@ module Weblog
 end
 
 class PolymorphicRoutesTest < ActionController::TestCase
-  include SharedTestRoutes.url_helpers
+  Routes = ActionDispatch::Routing::RouteSet.new
+  Routes.draw { }
+  include Routes.url_helpers
+
   default_url_options[:host] = "example.com"
 
   def setup
+    super
     @project = Project.new
     @task = Task.new
     @step = Step.new
@@ -460,12 +464,6 @@ class PolymorphicRoutesTest < ActionController::TestCase
     end
   end
 
-  def test_with_array_containing_single_string_name
-    with_test_routes do
-      assert_url "http://example.com/projects", ["projects"]
-    end
-  end
-
   def test_with_array_containing_symbols
     with_test_routes do
       assert_url "http://example.com/series/new", [:new, :series]
@@ -620,6 +618,22 @@ class PolymorphicRoutesTest < ActionController::TestCase
     end
   end
 
+  def test_string_route_arguments
+    with_admin_test_routes do
+      error = assert_raises(ArgumentError) do
+        polymorphic_url(["admin", @project])
+      end
+
+      assert_equal("Please use symbols for polymorphic route arguments.", error.message)
+
+      error = assert_raises(ArgumentError) do
+        polymorphic_url([@project, "bid"])
+      end
+
+      assert_equal("Please use symbols for polymorphic route arguments.", error.message)
+    end
+  end
+
   def with_namespaced_routes(name)
     with_routing do |set|
       set.draw do
@@ -763,9 +777,11 @@ class DirectRoutesTest < ActionView::TestCase
   include Routes.url_helpers
 
   def setup
+    super
     @category = Category.new("1")
     @collection = Collection.new("2")
     @product = Product.new("3")
+    @controller.singleton_class.include Routes.url_helpers
   end
 
   def test_direct_routes
