@@ -135,7 +135,7 @@ asyncTest('clicking on a link with both query string in href and data-params', 4
       App.assertGetRequest(data)
       equal(data.params.data1, 'value1', 'ajax arguments should have key data1 with right value')
       equal(data.params.data2, 'value2', 'ajax arguments should have key data2 with right value')
-      equal(data.params.data3, 'value3', 'query string in url should be passed to server with right value')
+      equal(data.params.data3, 'value3', 'query string in URL should be passed to server with right value')
     })
     .bindNative('ajax:complete', function() { start() })
     .triggerNative('click')
@@ -149,7 +149,7 @@ asyncTest('clicking on a link with both query string in href and data-params wit
       App.assertPostRequest(data)
       equal(data.params.data1, 'value1', 'ajax arguments should have key data1 with right value')
       equal(data.params.data2, 'value2', 'ajax arguments should have key data2 with right value')
-      equal(data.params.data3, 'value3', 'query string in url should be passed to server with right value')
+      equal(data.params.data3, 'value3', 'query string in URL should be passed to server with right value')
     })
     .bindNative('ajax:complete', function() { start() })
     .triggerNative('click')
@@ -490,4 +490,22 @@ asyncTest('changing a select option without "data-url" attribute still fires aja
   setTimeout(function() { start() }, 20)
 })
 
+asyncTest('inputs inside disabled fieldset are not submited on remote forms', 3, function() {
+  $('form')
+    .append('<fieldset>\
+      <input name="description" value="A wise man" />\
+    </fieldset>')
+    .append('<fieldset disabled="disabled">\
+      <input name="age" />\
+    </fieldset>')
+    .bindNative('ajax:success', function(e, data, status, xhr) {
+      equal(data.params.user_name, 'john')
+      equal(data.params.description, 'A wise man')
+      equal(data.params.age, undefined)
+
+      start()
+    })
+    .triggerNative('submit')
+})
+
 })()
