@@ -1,22 +1,23 @@
 # frozen_string_literal: true
 
 require "queue_classic"
+require "active_support/core_ext/module/redefine_method"
 
 module QC
   class Queue
-    def enqueue(method, *args)
+    redefine_method(:enqueue) do |method, *args|
       receiver_str, _, message = method.rpartition(".")
       receiver = eval(receiver_str)
       receiver.send(message, *args)
     end
 
-    def enqueue_in(seconds, method, *args)
+    redefine_method(:enqueue_in) do |seconds, method, *args|
       receiver_str, _, message = method.rpartition(".")
       receiver = eval(receiver_str)
       receiver.send(message, *args)
     end
 
-    def enqueue_at(not_before, method, *args)
+    redefine_method(:enqueue_at) do |not_before, method, *args|
       receiver_str, _, message = method.rpartition(".")
       receiver = eval(receiver_str)
       receiver.send(message, *args)
