@@ -14,37 +14,23 @@ class DirtyTest < ActiveModel::TestCase
       @status = "initialized"
     end
 
-    def name
-      @name
-    end
+    attr_reader :name, :color, :size, :status
 
     def name=(val)
       name_will_change!
       @name = val
     end
 
-    def color
-      @color
-    end
-
     def color=(val)
       color_will_change! unless val == @color
       @color = val
     end
 
-    def size
-      @size
-    end
-
     def size=(val)
       attribute_will_change!(:size) unless val == @size
       @size = val
     end
 
-    def status
-      @status
-    end
-
     def status=(val)
       status_will_change! unless val == @status
       @status = val
@@ -78,7 +64,7 @@ class DirtyTest < ActiveModel::TestCase
   end
 
   test "changes to attribute values" do
-    assert !@model.changes["name"]
+    assert_not @model.changes["name"]
     @model.name = "John"
     assert_equal [nil, "John"], @model.changes["name"]
   end
@@ -108,7 +94,7 @@ class DirtyTest < ActiveModel::TestCase
   end
 
   test "attribute mutation" do
-    @model.instance_variable_set("@name", "Yam".dup)
+    @model.instance_variable_set("@name", +"Yam")
     assert_not_predicate @model, :name_changed?
     @model.name.replace("Hadad")
     assert_not_predicate @model, :name_changed?
