@@ -6,12 +6,8 @@ require "securerandom"
 
 class SQLite3QuotingTest < ActiveRecord::SQLite3TestCase
   def setup
+    super
     @conn = ActiveRecord::Base.connection
-    @initial_represent_boolean_as_integer = ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer
-  end
-
-  def teardown
-    ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer = @initial_represent_boolean_as_integer
   end
 
   def test_type_cast_binary_encoding_without_logger
@@ -22,18 +18,10 @@ class SQLite3QuotingTest < ActiveRecord::SQLite3TestCase
   end
 
   def test_type_cast_true
-    ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer = false
-    assert_equal "t", @conn.type_cast(true)
-
-    ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer = true
     assert_equal 1, @conn.type_cast(true)
   end
 
   def test_type_cast_false
-    ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer = false
-    assert_equal "f", @conn.type_cast(false)
-
-    ActiveRecord::ConnectionAdapters::SQLite3Adapter.represent_boolean_as_integer = true
     assert_equal 0, @conn.type_cast(false)
   end
 
