@@ -9,7 +9,8 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
     @original_verbose = ActiveRecord::Migration.verbose
     ActiveRecord::Migration.verbose = false
     @connection = ActiveRecord::Base.connection
-    ActiveRecord::SchemaMigration.drop_table
+    @schema_migration = @connection.schema_migration
+    @schema_migration.drop_table
   end
 
   teardown do
@@ -18,21 +19,21 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
     @connection.drop_table :nep_schema_migrations rescue nil
     @connection.drop_table :has_timestamps rescue nil
     @connection.drop_table :multiple_indexes rescue nil
-    ActiveRecord::SchemaMigration.delete_all rescue nil
+    @schema_migration.delete_all rescue nil
     ActiveRecord::Migration.verbose = @original_verbose
   end
 
   def test_has_primary_key
     old_primary_key_prefix_type = ActiveRecord::Base.primary_key_prefix_type
     ActiveRecord::Base.primary_key_prefix_type = :table_name_with_underscore
-    assert_equal "version", ActiveRecord::SchemaMigration.primary_key
+    assert_equal "version", @schema_migration.primary_key
 
-    ActiveRecord::SchemaMigration.create_table
-    assert_difference "ActiveRecord::SchemaMigration.count", 1 do
-      ActiveRecord::SchemaMigration.create version: 12
+    @schema_migration.create_table
+    assert_difference "@schema_migration.count", 1 do
+      @schema_migration.create version: 12
     end
   ensure
-    ActiveRecord::SchemaMigration.drop_table
+    @schema_migration.drop_table
     ActiveRecord::Base.primary_key_prefix_type = old_primary_key_prefix_type
   end
 
@@ -51,11 +52,11 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
     assert_equal 7, @connection.migration_context.current_version
   end
 
-  def test_schema_define_w_table_name_prefix
-    table_name = ActiveRecord::SchemaMigration.table_name
+  def test_schema_define_with_table_name_prefix
     old_table_name_prefix = ActiveRecord::Base.table_name_prefix
     ActiveRecord::Base.table_name_prefix = "nep_"
-    ActiveRecord::SchemaMigration.table_name = "nep_#{table_name}"
+    @schema_migration.reset_table_name
+    ActiveRecord::InternalMetadata.reset_table_name
     ActiveRecord::Schema.define(version: 7) do
       create_table :fruits do |t|
         t.column :color, :string
@@ -67,7 +68,8 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
     assert_equal 7, @connection.migration_context.current_version
   ensure
     ActiveRecord::Base.table_name_prefix = old_table_name_prefix
-    ActiveRecord::SchemaMigration.table_name = table_name
+    @schema_migration.reset_table_name
+    ActiveRecord::InternalMetadata.reset_table_name
   end
 
   def test_schema_raises_an_error_for_invalid_column_type
@@ -88,10 +90,10 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
   end
 
   def test_normalize_version
-    assert_equal "118", ActiveRecord::SchemaMigration.normalize_migration_number("0000118")
-    assert_equal "002", ActiveRecord::SchemaMigration.normalize_migration_number("2")
-    assert_equal "017", ActiveRecord::SchemaMigration.normalize_migration_number("0017")
-    assert_equal "20131219224947", ActiveRecord::SchemaMigration.normalize_migration_number("20131219224947")
+    assert_equal "118", @schema_migration.normalize_migration_number("0000118")
+    assert_equal "002", @schema_migration.normalize_migration_number("2")
+    assert_equal "017", @schema_migration.normalize_migration_number("0017")
+    assert_equal "20131219224947", @schema_migration.normalize_migration_number("20131219224947")
   end
 
   def test_schema_load_with_multiple_indexes_for_column_of_different_names
@@ -116,8 +118,8 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
       end
     end
 
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "created_at" }.null
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "updated_at" }.null
+    assert @connection.column_exists?(:has_timestamps, :created_at, null: false)
+    assert @connection.column_exists?(:has_timestamps, :updated_at, null: false)
   end
 
   def test_timestamps_without_null_set_null_to_false_on_change_table
@@ -129,8 +131,23 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
       end
     end
 
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "created_at" }.null
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "updated_at" }.null
+    assert @connection.column_exists?(:has_timestamps, :created_at, null: false)
+    assert @connection.column_exists?(:has_timestamps, :updated_at, null: false)
+  end
+
+  if ActiveRecord::Base.connection.supports_bulk_alter?
+    def test_timestamps_without_null_set_null_to_false_on_change_table_with_bulk
+      ActiveRecord::Schema.define do
+        create_table :has_timestamps
+
+        change_table :has_timestamps, bulk: true do |t|
+          t.timestamps default: Time.now
+        end
+      end
+
+      assert @connection.column_exists?(:has_timestamps, :created_at, null: false)
+      assert @connection.column_exists?(:has_timestamps, :updated_at, null: false)
+    end
   end
 
   def test_timestamps_without_null_set_null_to_false_on_add_timestamps
@@ -139,7 +156,58 @@ class ActiveRecordSchemaTest < ActiveRecord::TestCase
       add_timestamps :has_timestamps, default: Time.now
     end
 
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "created_at" }.null
-    assert !@connection.columns(:has_timestamps).find { |c| c.name == "updated_at" }.null
+    assert @connection.column_exists?(:has_timestamps, :created_at, null: false)
+    assert @connection.column_exists?(:has_timestamps, :updated_at, null: false)
+  end
+
+  if subsecond_precision_supported?
+    def test_timestamps_sets_precision_on_create_table
+      ActiveRecord::Schema.define do
+        create_table :has_timestamps do |t|
+          t.timestamps
+        end
+      end
+
+      assert @connection.column_exists?(:has_timestamps, :created_at, precision: 6, null: false)
+      assert @connection.column_exists?(:has_timestamps, :updated_at, precision: 6, null: false)
+    end
+
+    def test_timestamps_sets_precision_on_change_table
+      ActiveRecord::Schema.define do
+        create_table :has_timestamps
+
+        change_table :has_timestamps do |t|
+          t.timestamps default: Time.now
+        end
+      end
+
+      assert @connection.column_exists?(:has_timestamps, :created_at, precision: 6, null: false)
+      assert @connection.column_exists?(:has_timestamps, :updated_at, precision: 6, null: false)
+    end
+
+    if ActiveRecord::Base.connection.supports_bulk_alter?
+      def test_timestamps_sets_precision_on_change_table_with_bulk
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps
+
+          change_table :has_timestamps, bulk: true do |t|
+            t.timestamps default: Time.now
+          end
+        end
+
+        assert @connection.column_exists?(:has_timestamps, :created_at, precision: 6, null: false)
+        assert @connection.column_exists?(:has_timestamps, :updated_at, precision: 6, null: false)
+      end
+    end
+
+    def test_timestamps_sets_precision_on_add_timestamps
+      ActiveRecord::Schema.define do
+        create_table :has_timestamps
+        add_timestamps :has_timestamps, default: Time.now
+      end
+
+      assert @connection.column_exists?(:has_timestamps, :created_at, precision: 6, null: false)
+      assert @connection.column_exists?(:has_timestamps, :updated_at, precision: 6, null: false)
+    end
   end
 end
