@@ -12,7 +12,7 @@ module ActiveRecord
       cloned = topic.clone
       assert topic.persisted?, "topic persisted"
       assert cloned.persisted?, "topic persisted"
-      assert !cloned.new_record?, "topic is not new"
+      assert_not cloned.new_record?, "topic is not new"
     end
 
     def test_stays_frozen
@@ -21,7 +21,7 @@ module ActiveRecord
 
       cloned = topic.clone
       assert cloned.persisted?, "topic persisted"
-      assert !cloned.new_record?, "topic is not new"
+      assert_not cloned.new_record?, "topic is not new"
       assert cloned.frozen?, "topic should be frozen"
     end
 
