@@ -2,7 +2,6 @@
 
 require "cases/helper"
 require "models/car"
-require "active_support/core_ext/string/strip"
 
 if ActiveRecord::Base.connection.supports_explain?
   class ExplainTest < ActiveRecord::TestCase
@@ -53,7 +52,7 @@ if ActiveRecord::Base.connection.supports_explain?
       queries = sqls.zip(binds)
 
       stub_explain_for_query_plans(["query plan foo\n", "query plan bar\n"]) do
-        expected = <<-SQL.strip_heredoc
+        expected = <<~SQL
           EXPLAIN for: #{sqls[0]} [["wadus", 1]]
           query plan foo
 
@@ -73,7 +72,6 @@ if ActiveRecord::Base.connection.supports_explain?
     end
 
     private
-
       def stub_explain_for_query_plans(query_plans = ["query plan foo", "query plan bar"])
         explain_called = 0
 
