@@ -3,7 +3,7 @@
 require "cases/helper"
 require "support/schema_dumping_helper"
 
-if ActiveRecord::Base.connection.supports_foreign_keys_in_create?
+if ActiveRecord::Base.connection.supports_foreign_keys?
   module ActiveRecord
     class Migration
       class ForeignKeyInCreateTest < ActiveRecord::TestCase
@@ -19,11 +19,152 @@ if ActiveRecord::Base.connection.supports_foreign_keys_in_create?
           assert_equal "fk_name", fk.name unless current_adapter?(:SQLite3Adapter)
         end
       end
+
+      class ForeignKeyChangeColumnTest < ActiveRecord::TestCase
+        self.use_transactional_tests = false
+
+        class Rocket < ActiveRecord::Base
+          has_many :astronauts
+        end
+
+        class Astronaut < ActiveRecord::Base
+          belongs_to :rocket
+        end
+
+        class CreateRocketsMigration < ActiveRecord::Migration::Current
+          def up
+            create_table :rockets do |t|
+              t.string :name
+            end
+
+            create_table :astronauts do |t|
+              t.string :name
+              t.references :rocket, foreign_key: true
+            end
+          end
+
+          def down
+            drop_table :astronauts, if_exists: true
+            drop_table :rockets, if_exists: true
+          end
+        end
+
+        def setup
+          @connection = ActiveRecord::Base.connection
+          @migration = CreateRocketsMigration.new
+          silence_stream($stdout) { @migration.migrate(:up) }
+          Rocket.reset_table_name
+          Rocket.reset_column_information
+          Astronaut.reset_table_name
+          Astronaut.reset_column_information
+        end
+
+        def teardown
+          silence_stream($stdout) { @migration.migrate(:down) }
+          Rocket.reset_table_name
+          Rocket.reset_column_information
+          Astronaut.reset_table_name
+          Astronaut.reset_column_information
+        end
+
+        def test_change_column_of_parent_table
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.change_column_null Rocket.table_name, :name, false
+
+          foreign_keys = @connection.foreign_keys(Astronaut.table_name)
+          assert_equal 1, foreign_keys.size
+
+          fk = foreign_keys.first
+          assert_equal "myrocket", Rocket.first.name
+          assert_equal Astronaut.table_name, fk.from_table
+          assert_equal Rocket.table_name, fk.to_table
+        end
+
+        def test_rename_column_of_child_table
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.rename_column Astronaut.table_name, :name, :astronaut_name
+
+          foreign_keys = @connection.foreign_keys(Astronaut.table_name)
+          assert_equal 1, foreign_keys.size
+
+          fk = foreign_keys.first
+          assert_equal "myrocket", Rocket.first.name
+          assert_equal Astronaut.table_name, fk.from_table
+          assert_equal Rocket.table_name, fk.to_table
+        end
+
+        def test_rename_reference_column_of_child_table
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.rename_column Astronaut.table_name, :rocket_id, :new_rocket_id
+
+          foreign_keys = @connection.foreign_keys(Astronaut.table_name)
+          assert_equal 1, foreign_keys.size
+
+          fk = foreign_keys.first
+          assert_equal "myrocket", Rocket.first.name
+          assert_equal Astronaut.table_name, fk.from_table
+          assert_equal Rocket.table_name, fk.to_table
+          assert_equal "new_rocket_id", fk.options[:column]
+        end
+
+        def test_remove_reference_column_of_child_table
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.remove_column Astronaut.table_name, :rocket_id
+
+          assert_empty @connection.foreign_keys(Astronaut.table_name)
+        end
+
+        def test_remove_foreign_key_by_column
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.remove_foreign_key Astronaut.table_name, column: :rocket_id
+
+          assert_empty @connection.foreign_keys(Astronaut.table_name)
+        end
+
+        def test_remove_foreign_key_by_column_in_change_table
+          rocket = Rocket.create!(name: "myrocket")
+          rocket.astronauts << Astronaut.create!
+
+          @connection.change_table Astronaut.table_name do |t|
+            t.remove_foreign_key column: :rocket_id
+          end
+
+          assert_empty @connection.foreign_keys(Astronaut.table_name)
+        end
+      end
+
+      class ForeignKeyChangeColumnWithPrefixTest < ForeignKeyChangeColumnTest
+        setup do
+          ActiveRecord::Base.table_name_prefix = "p_"
+        end
+
+        teardown do
+          ActiveRecord::Base.table_name_prefix = nil
+        end
+      end
+
+      class ForeignKeyChangeColumnWithSuffixTest < ForeignKeyChangeColumnTest
+        setup do
+          ActiveRecord::Base.table_name_suffix = "_s"
+        end
+
+        teardown do
+          ActiveRecord::Base.table_name_suffix = nil
+        end
+      end
     end
   end
-end
 
-if ActiveRecord::Base.connection.supports_foreign_keys?
   module ActiveRecord
     class Migration
       class ForeignKeyTest < ActiveRecord::TestCase
@@ -62,7 +203,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
           assert_equal "fk_test_has_pk", fk.to_table
           assert_equal "fk_id", fk.column
           assert_equal "pk_id", fk.primary_key
-          assert_equal "fk_name", fk.name
+          assert_equal "fk_name", fk.name unless current_adapter?(:SQLite3Adapter)
         end
 
         def test_add_foreign_key_inferes_column
@@ -76,7 +217,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
           assert_equal "rockets", fk.to_table
           assert_equal "rocket_id", fk.column
           assert_equal "id", fk.primary_key
-          assert_equal("fk_rails_78146ddd2e", fk.name)
+          assert_equal "fk_rails_78146ddd2e", fk.name unless current_adapter?(:SQLite3Adapter)
         end
 
         def test_add_foreign_key_with_column
@@ -90,7 +231,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
           assert_equal "rockets", fk.to_table
           assert_equal "rocket_id", fk.column
           assert_equal "id", fk.primary_key
-          assert_equal("fk_rails_78146ddd2e", fk.name)
+          assert_equal "fk_rails_78146ddd2e", fk.name unless current_adapter?(:SQLite3Adapter)
         end
 
         def test_add_foreign_key_with_non_standard_primary_key
@@ -109,7 +250,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
           assert_equal "space_shuttles", fk.to_table
           assert_equal "pk", fk.primary_key
         ensure
-          @connection.remove_foreign_key :astronauts, name: "custom_pk"
+          @connection.remove_foreign_key :astronauts, name: "custom_pk", to_table: "space_shuttles"
           @connection.drop_table :space_shuttles
         end
 
@@ -183,12 +324,28 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         def test_foreign_key_exists_by_name
+          skip if current_adapter?(:SQLite3Adapter)
+
           @connection.add_foreign_key :astronauts, :rockets, column: "rocket_id", name: "fancy_named_fk"
 
           assert @connection.foreign_key_exists?(:astronauts, name: "fancy_named_fk")
           assert_not @connection.foreign_key_exists?(:astronauts, name: "other_fancy_named_fk")
         end
 
+        def test_foreign_key_exists_in_change_table
+          @connection.change_table(:astronauts) do |t|
+            t.foreign_key :rockets, column: "rocket_id", name: "fancy_named_fk"
+
+            assert t.foreign_key_exists?(column: "rocket_id")
+            assert_not t.foreign_key_exists?(column: "star_id")
+
+            unless current_adapter?(:SQLite3Adapter)
+              assert t.foreign_key_exists?(name: "fancy_named_fk")
+              assert_not t.foreign_key_exists?(name: "other_fancy_named_fk")
+            end
+          end
+        end
+
         def test_remove_foreign_key_inferes_column
           @connection.add_foreign_key :astronauts, :rockets
 
@@ -214,6 +371,8 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         def test_remove_foreign_key_by_name
+          skip if current_adapter?(:SQLite3Adapter)
+
           @connection.add_foreign_key :astronauts, :rockets, column: "rocket_id", name: "fancy_named_fk"
 
           assert_equal 1, @connection.foreign_keys("astronauts").size
@@ -222,9 +381,22 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         def test_remove_foreign_non_existing_foreign_key_raises
-          assert_raises ArgumentError do
+          e = assert_raises ArgumentError do
             @connection.remove_foreign_key :astronauts, :rockets
           end
+          assert_equal "Table 'astronauts' has no foreign key for rockets", e.message
+        end
+
+        def test_remove_foreign_key_by_the_select_one_on_the_same_table
+          @connection.add_foreign_key :astronauts, :rockets
+          @connection.add_reference :astronauts, :myrocket, foreign_key: { to_table: :rockets }
+
+          assert_equal 2, @connection.foreign_keys("astronauts").size
+
+          @connection.remove_foreign_key :astronauts, :rockets, column: "myrocket_id"
+
+          assert_equal [["astronauts", "rockets", "rocket_id"]],
+            @connection.foreign_keys("astronauts").map { |fk| [fk.from_table, fk.to_table, fk.column] }
         end
 
         if ActiveRecord::Base.connection.supports_validate_constraints?
@@ -303,7 +475,22 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
 
         def test_schema_dumping_with_options
           output = dump_table_schema "fk_test_has_fk"
-          assert_match %r{\s+add_foreign_key "fk_test_has_fk", "fk_test_has_pk", column: "fk_id", primary_key: "pk_id", name: "fk_name"$}, output
+          if current_adapter?(:SQLite3Adapter)
+            assert_match %r{\s+add_foreign_key "fk_test_has_fk", "fk_test_has_pk", column: "fk_id", primary_key: "pk_id"$}, output
+          else
+            assert_match %r{\s+add_foreign_key "fk_test_has_fk", "fk_test_has_pk", column: "fk_id", primary_key: "pk_id", name: "fk_name"$}, output
+          end
+        end
+
+        def test_schema_dumping_with_custom_fk_ignore_pattern
+          original_pattern = ActiveRecord::SchemaDumper.fk_ignore_pattern
+          ActiveRecord::SchemaDumper.fk_ignore_pattern = /^ignored_/
+          @connection.add_foreign_key :astronauts, :rockets, name: :ignored_fk_astronauts_rockets
+
+          output = dump_table_schema "astronauts"
+          assert_match %r{\s+add_foreign_key "astronauts", "rockets"$}, output
+
+          ActiveRecord::SchemaDumper.fk_ignore_pattern = original_pattern
         end
 
         def test_schema_dumping_on_delete_and_on_update_options
@@ -346,7 +533,7 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
         end
 
         class CreateSchoolsAndClassesMigration < ActiveRecord::Migration::Current
-          def change
+          def up
             create_table(:schools)
 
             create_table(:classes) do |t|
@@ -354,6 +541,11 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
             end
             add_foreign_key :classes, :schools
           end
+
+          def down
+            drop_table :classes, if_exists: true
+            drop_table :schools, if_exists: true
+          end
         end
 
         def test_add_foreign_key_with_prefix
@@ -378,30 +570,4 @@ if ActiveRecord::Base.connection.supports_foreign_keys?
       end
     end
   end
-else
-  module ActiveRecord
-    class Migration
-      class NoForeignKeySupportTest < ActiveRecord::TestCase
-        setup do
-          @connection = ActiveRecord::Base.connection
-        end
-
-        def test_add_foreign_key_should_be_noop
-          @connection.add_foreign_key :clubs, :categories
-        end
-
-        def test_remove_foreign_key_should_be_noop
-          @connection.remove_foreign_key :clubs, :categories
-        end
-
-        unless current_adapter?(:SQLite3Adapter)
-          def test_foreign_keys_should_raise_not_implemented
-            assert_raises NotImplementedError do
-              @connection.foreign_keys("clubs")
-            end
-          end
-        end
-      end
-    end
-  end
 end
