@@ -23,8 +23,9 @@ class MigratorTest < ActiveRecord::TestCase
 
   def setup
     super
-    ActiveRecord::SchemaMigration.create_table
-    ActiveRecord::SchemaMigration.delete_all rescue nil
+    @schema_migration = ActiveRecord::Base.connection.schema_migration
+    @schema_migration.create_table
+    @schema_migration.delete_all rescue nil
     @verbose_was = ActiveRecord::Migration.verbose
     ActiveRecord::Migration.message_count = 0
     ActiveRecord::Migration.class_eval do
@@ -36,7 +37,7 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   teardown do
-    ActiveRecord::SchemaMigration.delete_all rescue nil
+    @schema_migration.delete_all rescue nil
     ActiveRecord::Migration.verbose = @verbose_was
     ActiveRecord::Migration.class_eval do
       undef :puts
@@ -49,7 +50,7 @@ class MigratorTest < ActiveRecord::TestCase
   def test_migrator_with_duplicate_names
     e = assert_raises(ActiveRecord::DuplicateMigrationNameError) do
       list = [ActiveRecord::Migration.new("Chunky"), ActiveRecord::Migration.new("Chunky")]
-      ActiveRecord::Migrator.new(:up, list)
+      ActiveRecord::Migrator.new(:up, list, @schema_migration)
     end
     assert_match(/Multiple migrations have the name Chunky/, e.message)
   end
@@ -57,39 +58,40 @@ class MigratorTest < ActiveRecord::TestCase
   def test_migrator_with_duplicate_versions
     assert_raises(ActiveRecord::DuplicateMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 1)]
-      ActiveRecord::Migrator.new(:up, list)
+      ActiveRecord::Migrator.new(:up, list, @schema_migration)
     end
   end
 
   def test_migrator_with_missing_version_numbers
     assert_raises(ActiveRecord::UnknownMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 2)]
-      ActiveRecord::Migrator.new(:up, list, 3).run
+      ActiveRecord::Migrator.new(:up, list, @schema_migration, 3).run
     end
 
     assert_raises(ActiveRecord::UnknownMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 2)]
-      ActiveRecord::Migrator.new(:up, list, -1).run
+      ActiveRecord::Migrator.new(:up, list, @schema_migration, -1).run
     end
 
     assert_raises(ActiveRecord::UnknownMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 2)]
-      ActiveRecord::Migrator.new(:up, list, 0).run
+      ActiveRecord::Migrator.new(:up, list, @schema_migration, 0).run
     end
 
     assert_raises(ActiveRecord::UnknownMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 2)]
-      ActiveRecord::Migrator.new(:up, list, 3).migrate
+      ActiveRecord::Migrator.new(:up, list, @schema_migration, 3).migrate
     end
 
     assert_raises(ActiveRecord::UnknownMigrationVersionError) do
       list = [ActiveRecord::Migration.new("Foo", 1), ActiveRecord::Migration.new("Bar", 2)]
-      ActiveRecord::Migrator.new(:up, list, -1).migrate
+      ActiveRecord::Migrator.new(:up, list, @schema_migration, -1).migrate
     end
   end
 
   def test_finds_migrations
-    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/valid").migrations
+    schema_migration = ActiveRecord::Base.connection.schema_migration
+    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/valid", schema_migration).migrations
 
     [[1, "ValidPeopleHaveLastNames"], [2, "WeNeedReminders"], [3, "InnocentJointable"]].each_with_index do |pair, i|
       assert_equal migrations[i].version, pair.first
@@ -98,8 +100,8 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_finds_migrations_in_subdirectories
-    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/valid_with_subdirectories").migrations
-
+    schema_migration = ActiveRecord::Base.connection.schema_migration
+    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/valid_with_subdirectories", schema_migration).migrations
 
     [[1, "ValidPeopleHaveLastNames"], [2, "WeNeedReminders"], [3, "InnocentJointable"]].each_with_index do |pair, i|
       assert_equal migrations[i].version, pair.first
@@ -108,8 +110,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_finds_migrations_from_two_directories
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     directories = [MIGRATIONS_ROOT + "/valid_with_timestamps", MIGRATIONS_ROOT + "/to_copy_with_timestamps"]
-    migrations = ActiveRecord::MigrationContext.new(directories).migrations
+    migrations = ActiveRecord::MigrationContext.new(directories, schema_migration).migrations
 
     [[20090101010101, "PeopleHaveHobbies"],
      [20090101010202, "PeopleHaveDescriptions"],
@@ -122,14 +125,16 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_finds_migrations_in_numbered_directory
-    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/10_urban").migrations
+    schema_migration = ActiveRecord::Base.connection.schema_migration
+    migrations = ActiveRecord::MigrationContext.new(MIGRATIONS_ROOT + "/10_urban", schema_migration).migrations
     assert_equal 9, migrations[0].version
     assert_equal "AddExpressions", migrations[0].name
   end
 
   def test_relative_migrations
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     list = Dir.chdir(MIGRATIONS_ROOT) do
-      ActiveRecord::MigrationContext.new("valid").migrations
+      ActiveRecord::MigrationContext.new("valid", schema_migration).migrations
     end
 
     migration_proxy = list.find { |item|
@@ -139,9 +144,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_finds_pending_migrations
-    ActiveRecord::SchemaMigration.create!(version: "1")
+    @schema_migration.create!(version: "1")
     migration_list = [ActiveRecord::Migration.new("foo", 1), ActiveRecord::Migration.new("bar", 3)]
-    migrations = ActiveRecord::Migrator.new(:up, migration_list).pending_migrations
+    migrations = ActiveRecord::Migrator.new(:up, migration_list, @schema_migration).pending_migrations
 
     assert_equal 1, migrations.size
     assert_equal migration_list.last, migrations.first
@@ -149,35 +154,38 @@ class MigratorTest < ActiveRecord::TestCase
 
   def test_migrations_status
     path = MIGRATIONS_ROOT + "/valid"
+    schema_migration = ActiveRecord::Base.connection.schema_migration
 
-    ActiveRecord::SchemaMigration.create(version: 2)
-    ActiveRecord::SchemaMigration.create(version: 10)
+    @schema_migration.create(version: 2)
+    @schema_migration.create(version: 10)
 
     assert_equal [
       ["down", "001", "Valid people have last names"],
       ["up",   "002", "We need reminders"],
       ["down", "003", "Innocent jointable"],
       ["up",   "010", "********** NO FILE **********"],
-    ], ActiveRecord::MigrationContext.new(path).migrations_status
+    ], ActiveRecord::MigrationContext.new(path, schema_migration).migrations_status
   end
 
   def test_migrations_status_in_subdirectories
     path = MIGRATIONS_ROOT + "/valid_with_subdirectories"
+    schema_migration = ActiveRecord::Base.connection.schema_migration
 
-    ActiveRecord::SchemaMigration.create(version: 2)
-    ActiveRecord::SchemaMigration.create(version: 10)
+    @schema_migration.create(version: 2)
+    @schema_migration.create(version: 10)
 
     assert_equal [
       ["down", "001", "Valid people have last names"],
       ["up",   "002", "We need reminders"],
       ["down", "003", "Innocent jointable"],
       ["up",   "010", "********** NO FILE **********"],
-    ], ActiveRecord::MigrationContext.new(path).migrations_status
+    ], ActiveRecord::MigrationContext.new(path, schema_migration).migrations_status
   end
 
   def test_migrations_status_with_schema_define_in_subdirectories
     path = MIGRATIONS_ROOT + "/valid_with_subdirectories"
     prev_paths = ActiveRecord::Migrator.migrations_paths
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     ActiveRecord::Migrator.migrations_paths = path
 
     ActiveRecord::Schema.define(version: 3) do
@@ -187,16 +195,17 @@ class MigratorTest < ActiveRecord::TestCase
       ["up", "001", "Valid people have last names"],
       ["up", "002", "We need reminders"],
       ["up", "003", "Innocent jointable"],
-    ], ActiveRecord::MigrationContext.new(path).migrations_status
+    ], ActiveRecord::MigrationContext.new(path, schema_migration).migrations_status
   ensure
     ActiveRecord::Migrator.migrations_paths = prev_paths
   end
 
   def test_migrations_status_from_two_directories
     paths = [MIGRATIONS_ROOT + "/valid_with_timestamps", MIGRATIONS_ROOT + "/to_copy_with_timestamps"]
+    schema_migration = ActiveRecord::Base.connection.schema_migration
 
-    ActiveRecord::SchemaMigration.create(version: "20100101010101")
-    ActiveRecord::SchemaMigration.create(version: "20160528010101")
+    @schema_migration.create(version: "20100101010101")
+    @schema_migration.create(version: "20160528010101")
 
     assert_equal [
       ["down", "20090101010101", "People have hobbies"],
@@ -205,18 +214,18 @@ class MigratorTest < ActiveRecord::TestCase
       ["down", "20100201010101", "Valid with timestamps we need reminders"],
       ["down", "20100301010101", "Valid with timestamps innocent jointable"],
       ["up",   "20160528010101", "********** NO FILE **********"],
-    ], ActiveRecord::MigrationContext.new(paths).migrations_status
+    ], ActiveRecord::MigrationContext.new(paths, schema_migration).migrations_status
   end
 
   def test_migrator_interleaved_migrations
     pass_one = [Sensor.new("One", 1)]
 
-    ActiveRecord::Migrator.new(:up, pass_one).migrate
+    ActiveRecord::Migrator.new(:up, pass_one, @schema_migration).migrate
     assert pass_one.first.went_up
     assert_not pass_one.first.went_down
 
     pass_two = [Sensor.new("One", 1), Sensor.new("Three", 3)]
-    ActiveRecord::Migrator.new(:up, pass_two).migrate
+    ActiveRecord::Migrator.new(:up, pass_two, @schema_migration).migrate
     assert_not pass_two[0].went_up
     assert pass_two[1].went_up
     assert pass_two.all? { |x| !x.went_down }
@@ -225,7 +234,7 @@ class MigratorTest < ActiveRecord::TestCase
                   Sensor.new("Two", 2),
                   Sensor.new("Three", 3)]
 
-    ActiveRecord::Migrator.new(:down, pass_three).migrate
+    ActiveRecord::Migrator.new(:down, pass_three, @schema_migration).migrate
     assert pass_three[0].went_down
     assert_not pass_three[1].went_down
     assert pass_three[2].went_down
@@ -233,7 +242,7 @@ class MigratorTest < ActiveRecord::TestCase
 
   def test_up_calls_up
     migrations = [Sensor.new(nil, 0), Sensor.new(nil, 1), Sensor.new(nil, 2)]
-    migrator = ActiveRecord::Migrator.new(:up, migrations)
+    migrator = ActiveRecord::Migrator.new(:up, migrations, @schema_migration)
     migrator.migrate
     assert migrations.all?(&:went_up)
     assert migrations.all? { |m| !m.went_down }
@@ -244,7 +253,7 @@ class MigratorTest < ActiveRecord::TestCase
     test_up_calls_up
 
     migrations = [Sensor.new(nil, 0), Sensor.new(nil, 1), Sensor.new(nil, 2)]
-    migrator = ActiveRecord::Migrator.new(:down, migrations)
+    migrator = ActiveRecord::Migrator.new(:down, migrations, @schema_migration)
     migrator.migrate
     assert migrations.all? { |m| !m.went_up }
     assert migrations.all?(&:went_down)
@@ -252,30 +261,31 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_current_version
-    ActiveRecord::SchemaMigration.create!(version: "1000")
-    migrator = ActiveRecord::MigrationContext.new("db/migrate")
+    @schema_migration.create!(version: "1000")
+    schema_migration = ActiveRecord::Base.connection.schema_migration
+    migrator = ActiveRecord::MigrationContext.new("db/migrate", schema_migration)
     assert_equal 1000, migrator.current_version
   end
 
   def test_migrator_one_up
     calls, migrations = sensors(3)
 
-    ActiveRecord::Migrator.new(:up, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1).migrate
     assert_equal [[:up, 1]], calls
     calls.clear
 
-    ActiveRecord::Migrator.new(:up, migrations, 2).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 2).migrate
     assert_equal [[:up, 2]], calls
   end
 
   def test_migrator_one_down
     calls, migrations = sensors(3)
 
-    ActiveRecord::Migrator.new(:up, migrations).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration).migrate
     assert_equal [[:up, 1], [:up, 2], [:up, 3]], calls
     calls.clear
 
-    ActiveRecord::Migrator.new(:down, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 1).migrate
 
     assert_equal [[:down, 3], [:down, 2]], calls
   end
@@ -283,17 +293,17 @@ class MigratorTest < ActiveRecord::TestCase
   def test_migrator_one_up_one_down
     calls, migrations = sensors(3)
 
-    ActiveRecord::Migrator.new(:up, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1).migrate
     assert_equal [[:up, 1]], calls
     calls.clear
 
-    ActiveRecord::Migrator.new(:down, migrations, 0).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 0).migrate
     assert_equal [[:down, 1]], calls
   end
 
   def test_migrator_double_up
     calls, migrations = sensors(3)
-    migrator = ActiveRecord::Migrator.new(:up, migrations, 1)
+    migrator = ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1)
     assert_equal(0, migrator.current_version)
 
     migrator.migrate
@@ -306,7 +316,7 @@ class MigratorTest < ActiveRecord::TestCase
 
   def test_migrator_double_down
     calls, migrations = sensors(3)
-    migrator = ActiveRecord::Migrator.new(:up, migrations, 1)
+    migrator = ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1)
 
     assert_equal 0, migrator.current_version
 
@@ -314,7 +324,7 @@ class MigratorTest < ActiveRecord::TestCase
     assert_equal [[:up, 1]], calls
     calls.clear
 
-    migrator = ActiveRecord::Migrator.new(:down, migrations, 1)
+    migrator = ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 1)
     migrator.run
     assert_equal [[:down, 1]], calls
     calls.clear
@@ -329,12 +339,12 @@ class MigratorTest < ActiveRecord::TestCase
     _, migrations = sensors(3)
 
     ActiveRecord::Migration.verbose = true
-    ActiveRecord::Migrator.new(:up, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1).migrate
     assert_not_equal 0, ActiveRecord::Migration.message_count
 
     ActiveRecord::Migration.message_count = 0
 
-    ActiveRecord::Migrator.new(:down, migrations, 0).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 0).migrate
     assert_not_equal 0, ActiveRecord::Migration.message_count
   end
 
@@ -342,9 +352,9 @@ class MigratorTest < ActiveRecord::TestCase
     _, migrations = sensors(3)
 
     ActiveRecord::Migration.verbose = false
-    ActiveRecord::Migrator.new(:up, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1).migrate
     assert_equal 0, ActiveRecord::Migration.message_count
-    ActiveRecord::Migrator.new(:down, migrations, 0).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 0).migrate
     assert_equal 0, ActiveRecord::Migration.message_count
   end
 
@@ -352,23 +362,24 @@ class MigratorTest < ActiveRecord::TestCase
     calls, migrations = sensors(3)
 
     # migrate up to 1
-    ActiveRecord::Migrator.new(:up, migrations, 1).migrate
+    ActiveRecord::Migrator.new(:up, migrations, @schema_migration, 1).migrate
     assert_equal [[:up, 1]], calls
     calls.clear
 
     # migrate down to 0
-    ActiveRecord::Migrator.new(:down, migrations, 0).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 0).migrate
     assert_equal [[:down, 1]], calls
     calls.clear
 
     # migrate down to 0 again
-    ActiveRecord::Migrator.new(:down, migrations, 0).migrate
+    ActiveRecord::Migrator.new(:down, migrations, @schema_migration, 0).migrate
     assert_equal [], calls
   end
 
   def test_migrator_going_down_due_to_version_target
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     calls, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     migrator.up(1)
     assert_equal [[:up, 1]], calls
@@ -383,8 +394,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_migrator_output_when_running_multiple_migrations
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     result = migrator.migrate
     assert_equal(3, result.count)
@@ -398,8 +410,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_migrator_output_when_running_single_migration
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(1)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     result = migrator.run(:up, 1)
 
@@ -407,8 +420,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_migrator_rollback
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     migrator.migrate
     assert_equal(3, migrator.current_version)
@@ -427,8 +441,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_migrator_db_has_no_schema_migrations_table
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     ActiveRecord::Base.connection.drop_table "schema_migrations", if_exists: true
     assert_not ActiveRecord::Base.connection.table_exists?("schema_migrations")
@@ -437,8 +452,9 @@ class MigratorTest < ActiveRecord::TestCase
   end
 
   def test_migrator_forward
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(3)
-    migrator = migrator.new("/valid")
+    migrator = migrator.new("/valid", schema_migration)
     migrator.migrate(1)
     assert_equal(1, migrator.current_version)
 
@@ -451,18 +467,20 @@ class MigratorTest < ActiveRecord::TestCase
 
   def test_only_loads_pending_migrations
     # migrate up to 1
-    ActiveRecord::SchemaMigration.create!(version: "1")
+    @schema_migration.create!(version: "1")
 
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     calls, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
     migrator.migrate
 
     assert_equal [[:up, 2], [:up, 3]], calls
   end
 
   def test_get_all_versions
+    schema_migration = ActiveRecord::Base.connection.schema_migration
     _, migrator = migrator_class(3)
-    migrator = migrator.new("valid")
+    migrator = migrator.new("valid", schema_migration)
 
     migrator.migrate
     assert_equal([1, 2, 3], migrator.get_all_versions)
