@@ -1,18 +1,23 @@
 # frozen_string_literal: true
 
 require "cases/helper"
-require "models/topic"
-require "models/reply"
 require "models/person"
 require "models/traffic_light"
 require "models/post"
-require "bcrypt"
 
 class SerializedAttributeTest < ActiveRecord::TestCase
   fixtures :topics, :posts
 
   MyObject = Struct.new :attribute1, :attribute2
 
+  class Topic < ActiveRecord::Base
+    serialize :content
+  end
+
+  class ImportantTopic < Topic
+    serialize :important, Hash
+  end
+
   teardown do
     Topic.serialize("content")
   end
@@ -49,10 +54,10 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   def test_serialized_attributes_from_database_on_subclass
     Topic.serialize :content, Hash
 
-    t = Reply.new(content: { foo: :bar })
+    t = ImportantTopic.new(content: { foo: :bar })
     assert_equal({ foo: :bar }, t.content)
     t.save!
-    t = Reply.last
+    t = ImportantTopic.last
     assert_equal({ foo: :bar }, t.content)
   end
 
@@ -322,7 +327,7 @@ class SerializedAttributeTest < ActiveRecord::TestCase
     topic = Topic.create!(content: {})
     topic2 = Topic.create!(content: nil)
 
-    assert_equal [topic, topic2], Topic.where(content: nil)
+    assert_equal [topic, topic2], Topic.where(content: nil).sort_by(&:id)
   end
 
   def test_nil_is_always_persisted_as_null
@@ -367,9 +372,9 @@ class SerializedAttributeTest < ActiveRecord::TestCase
   end
 
   def test_serialized_attribute_works_under_concurrent_initial_access
-    model = Topic.dup
+    model = Class.new(Topic)
 
-    topic = model.last
+    topic = model.create!
     topic.update group: "1"
 
     model.serialize :group, JSON
@@ -377,9 +382,9 @@ class SerializedAttributeTest < ActiveRecord::TestCase
 
     # This isn't strictly necessary for the test, but a little bit of
     # knowledge of internals allows us to make failures far more likely.
-    model.define_singleton_method(:define_attribute) do |*args|
+    model.define_singleton_method(:define_attribute) do |*args, **kwargs|
       Thread.pass
-      super(*args)
+      super(*args, **kwargs)
     end
 
     threads = 4.times.map do
