@@ -4,6 +4,7 @@ require "cases/helper"
 require "models/book"
 require "models/liquid"
 require "models/molecule"
+require "models/numeric_data"
 require "models/electron"
 
 module ActiveRecord
@@ -74,6 +75,11 @@ module ActiveRecord
       assert_equal "salty", liquids[0].name
     end
 
+    def test_statement_cache_with_strictly_cast_attribute
+      row = NumericData.create(temperature: 1.5)
+      assert_equal row, NumericData.find_by(temperature: 1.5)
+    end
+
     def test_statement_cache_values_differ
       cache = ActiveRecord::StatementCache.create(Book.connection) do |params|
         Book.where(name: "my book")
@@ -102,7 +108,7 @@ module ActiveRecord
         Book.find_by(name: "my other book")
       end
 
-      refute_equal book, other_book
+      assert_not_equal book, other_book
     end
 
     def test_find_by_does_not_use_statement_cache_if_table_name_is_changed
