@@ -24,3 +24,9 @@ class Book < ActiveRecord::Base
     "do publish work..."
   end
 end
+
+class PublishedBook < ActiveRecord::Base
+  self.table_name = "books"
+
+  validates_uniqueness_of :isbn
+end
