@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 class MixedCaseMonkey < ActiveRecord::Base
-  self.primary_key = 'monkeyID'
+  belongs_to :man
 end
