@@ -6,6 +6,13 @@ class Reply < Topic
   belongs_to :topic, foreign_key: "parent_id", counter_cache: true
   belongs_to :topic_with_primary_key, class_name: "Topic", primary_key: "title", foreign_key: "parent_title", counter_cache: "replies_count", touch: true
   has_many :replies, class_name: "SillyReply", dependent: :destroy, foreign_key: "parent_id"
+  has_many :silly_unique_replies, dependent: :destroy, foreign_key: "parent_id"
+
+  scope :ordered, -> { Reply.order(:id) }
+end
+
+class SillyReply < Topic
+  belongs_to :reply, foreign_key: "parent_id", counter_cache: :replies_count
 end
 
 class UniqueReply < Reply
@@ -14,10 +21,6 @@ class UniqueReply < Reply
 end
 
 class SillyUniqueReply < UniqueReply
-end
-
-class ValidateUniqueContentReply < Reply
-  belongs_to :topic, foreign_key: "parent_id"
   validates :content, uniqueness: true
 end
 
@@ -57,10 +60,6 @@ class WrongReply < Reply
   end
 end
 
-class SillyReply < Reply
-  belongs_to :reply, foreign_key: "parent_id", counter_cache: :replies_count
-end
-
 module Web
   class Reply < Web::Topic
     belongs_to :topic, foreign_key: "parent_id", counter_cache: true, class_name: "Web::Topic"
