@@ -2,6 +2,7 @@
 
 require "service/shared_service_tests"
 require "net/http"
+require "database/setup"
 
 if SERVICE_CONFIGURATIONS[:s3]
   class ActiveStorage::Service::S3ServiceTest < ActiveSupport::TestCase
@@ -10,25 +11,23 @@ if SERVICE_CONFIGURATIONS[:s3]
     include ActiveStorage::Service::SharedServiceTests
 
     test "direct upload" do
-      begin
-        key      = SecureRandom.base58(24)
-        data     = "Something else entirely!"
-        checksum = Digest::MD5.base64digest(data)
-        url      = @service.url_for_direct_upload(key, expires_in: 5.minutes, content_type: "text/plain", content_length: data.size, checksum: checksum)
+      key      = SecureRandom.base58(24)
+      data     = "Something else entirely!"
+      checksum = Digest::MD5.base64digest(data)
+      url      = @service.url_for_direct_upload(key, expires_in: 5.minutes, content_type: "text/plain", content_length: data.size, checksum: checksum)
 
-        uri = URI.parse url
-        request = Net::HTTP::Put.new uri.request_uri
-        request.body = data
-        request.add_field "Content-Type", "text/plain"
-        request.add_field "Content-MD5", checksum
-        Net::HTTP.start(uri.host, uri.port, use_ssl: true) do |http|
-          http.request request
-        end
-
-        assert_equal data, @service.download(key)
-      ensure
-        @service.delete key
+      uri = URI.parse url
+      request = Net::HTTP::Put.new uri.request_uri
+      request.body = data
+      request.add_field "Content-Type", "text/plain"
+      request.add_field "Content-MD5", checksum
+      Net::HTTP.start(uri.host, uri.port, use_ssl: true) do |http|
+        http.request request
       end
+
+      assert_equal data, @service.download(key)
+    ensure
+      @service.delete key
     end
 
     test "directly uploading file larger than the provided content-length does not work" do
@@ -83,6 +82,24 @@ if SERVICE_CONFIGURATIONS[:s3]
         service.delete key
       end
     end
+
+    test "upload with content type" do
+      key          = SecureRandom.base58(24)
+      data         = "Something else entirely!"
+      content_type = "text/plain"
+
+      @service.upload(
+        key,
+        StringIO.new(data),
+        checksum: Digest::MD5.base64digest(data),
+        filename: "cool_data.txt",
+        content_type: content_type
+      )
+
+      assert_equal content_type, @service.bucket.object(key).content_type
+    ensure
+      @service.delete key
+    end
   end
 else
   puts "Skipping S3 Service tests because no S3 configuration was supplied"
