@@ -1,12 +1,13 @@
-require 'active_support/hash_with_indifferent_access'
+# frozen_string_literal: true
+
+require "active_support/hash_with_indifferent_access"
 
 class Hash
-
   # Returns an <tt>ActiveSupport::HashWithIndifferentAccess</tt> out of its receiver:
   #
   #   { a: 1 }.with_indifferent_access['a'] # => 1
   def with_indifferent_access
-    ActiveSupport::HashWithIndifferentAccess.new_from_hash_copying_default(self)
+    ActiveSupport::HashWithIndifferentAccess.new(self)
   end
 
   # Called when object is nested under an object that receives
@@ -18,5 +19,6 @@ class Hash
   #
   #   b = { b: 1 }
   #   { a: b }.with_indifferent_access['a'] # calls b.nested_under_indifferent_access
+  #   # => {"b"=>1}
   alias nested_under_indifferent_access with_indifferent_access
 end
