@@ -1,11 +1,13 @@
-require 'yaml'
+# frozen_string_literal: true
+
+require "yaml"
 
 YAML.add_builtin_type("omap") do |type, val|
-  ActiveSupport::OrderedHash[val.map{ |v| v.to_a.first }]
+  ActiveSupport::OrderedHash[val.map { |v| v.to_a.first }]
 end
 
 module ActiveSupport
-  # <tt>ActiveSupport::OrderedHash</tt> implements a hash that preserves
+  # DEPRECATED: <tt>ActiveSupport::OrderedHash</tt> implements a hash that preserves
   # insertion order.
   #
   #   oh = ActiveSupport::OrderedHash.new
@@ -14,7 +16,7 @@ module ActiveSupport
   #   oh.keys # => [:a, :b], this order is guaranteed
   #
   # Also, maps the +omap+ feature for YAML files
-  # (See http://yaml.org/type/omap.html) to support ordered items
+  # (See https://yaml.org/type/omap.html) to support ordered items
   # when loading from yaml.
   #
   # <tt>ActiveSupport::OrderedHash</tt> is namespaced to prevent conflicts
@@ -25,7 +27,15 @@ module ActiveSupport
     end
 
     def encode_with(coder)
-      coder.represent_seq '!omap', map { |k,v| { k => v } }
+      coder.represent_seq "!omap", map { |k, v| { k => v } }
+    end
+
+    def select(*args, &block)
+      dup.tap { |hash| hash.select!(*args, &block) }
+    end
+
+    def reject(*args, &block)
+      dup.tap { |hash| hash.reject!(*args, &block) }
     end
 
     def nested_under_indifferent_access
