@@ -128,4 +128,12 @@ class ConcernTest < ActiveSupport::TestCase
       end
     end
   end
+
+  def test_no_raise_on_same_included_call
+    assert_nothing_raised do
+      2.times do
+        load File.expand_path("../fixtures/concern/some_concern.rb", __FILE__)
+      end
+    end
+  end
 end
