@@ -112,14 +112,6 @@ class DateExtCalculationsTest < ActiveSupport::TestCase
     assert_equal Date.new(2005, 4, 30), Date.new(2005, 4, 20).end_of_month
   end
 
-  def test_prev_year_in_leap_years
-    assert_equal Date.new(1999, 2, 28), Date.new(2000, 2, 29).prev_year
-  end
-
-  def test_prev_year_in_calendar_reform
-    assert_equal Date.new(1582, 10, 4), Date.new(1583, 10, 14).prev_year
-  end
-
   def test_last_year_in_leap_years
     assert_equal Date.new(1999, 2, 28), Date.new(2000, 2, 29).last_year
   end
@@ -128,14 +120,6 @@ class DateExtCalculationsTest < ActiveSupport::TestCase
     assert_equal Date.new(1582, 10, 4), Date.new(1583, 10, 14).last_year
   end
 
-  def test_next_year_in_leap_years
-    assert_equal Date.new(2001, 2, 28), Date.new(2000, 2, 29).next_year
-  end
-
-  def test_next_year_in_calendar_reform
-    assert_equal Date.new(1582, 10, 4), Date.new(1581, 10, 10).next_year
-  end
-
   def test_advance
     assert_equal Date.new(2006, 2, 28), Date.new(2005, 2, 28).advance(years: 1)
     assert_equal Date.new(2005, 6, 28), Date.new(2005, 2, 28).advance(months: 4)
