@@ -31,10 +31,10 @@ class HashExtTest < ActiveSupport::TestCase
 
   def test_methods
     h = {}
-    assert_respond_to h, :transform_keys
-    assert_respond_to h, :transform_keys!
     assert_respond_to h, :deep_transform_keys
     assert_respond_to h, :deep_transform_keys!
+    assert_respond_to h, :deep_transform_values
+    assert_respond_to h, :deep_transform_values!
     assert_respond_to h, :symbolize_keys
     assert_respond_to h, :symbolize_keys!
     assert_respond_to h, :deep_symbolize_keys
@@ -45,24 +45,10 @@ class HashExtTest < ActiveSupport::TestCase
     assert_respond_to h, :deep_stringify_keys!
     assert_respond_to h, :to_options
     assert_respond_to h, :to_options!
-    assert_respond_to h, :compact
-    assert_respond_to h, :compact!
     assert_respond_to h, :except
     assert_respond_to h, :except!
   end
 
-  def test_transform_keys
-    assert_equal @upcase_strings, @strings.transform_keys { |key| key.to_s.upcase }
-    assert_equal @upcase_strings, @symbols.transform_keys { |key| key.to_s.upcase }
-    assert_equal @upcase_strings, @mixed.transform_keys { |key| key.to_s.upcase }
-  end
-
-  def test_transform_keys_not_mutates
-    transformed_hash = @mixed.dup
-    transformed_hash.transform_keys { |key| key.to_s.upcase }
-    assert_equal @mixed, transformed_hash
-  end
-
   def test_deep_transform_keys
     assert_equal @nested_upcase_strings, @nested_symbols.deep_transform_keys { |key| key.to_s.upcase }
     assert_equal @nested_upcase_strings, @nested_strings.deep_transform_keys { |key| key.to_s.upcase }
@@ -78,19 +64,6 @@ class HashExtTest < ActiveSupport::TestCase
     assert_equal @nested_mixed, transformed_hash
   end
 
-  def test_transform_keys!
-    assert_equal @upcase_strings, @symbols.dup.transform_keys! { |key| key.to_s.upcase }
-    assert_equal @upcase_strings, @strings.dup.transform_keys! { |key| key.to_s.upcase }
-    assert_equal @upcase_strings, @mixed.dup.transform_keys! { |key| key.to_s.upcase }
-  end
-
-  def test_transform_keys_with_bang_mutates
-    transformed_hash = @mixed.dup
-    transformed_hash.transform_keys! { |key| key.to_s.upcase }
-    assert_equal @upcase_strings, transformed_hash
-    assert_equal({ :a => 1, "b" => 2 }, @mixed)
-  end
-
   def test_deep_transform_keys!
     assert_equal @nested_upcase_strings, @nested_symbols.deep_dup.deep_transform_keys! { |key| key.to_s.upcase }
     assert_equal @nested_upcase_strings, @nested_strings.deep_dup.deep_transform_keys! { |key| key.to_s.upcase }
@@ -107,6 +80,31 @@ class HashExtTest < ActiveSupport::TestCase
     assert_equal({ "a" => { b: { "c" => 3 } } }, @nested_mixed)
   end
 
+  def test_deep_transform_values
+    assert_equal({ "a" => "1", "b" => "2" }, @strings.deep_transform_values { |value| value.to_s })
+    assert_equal({ "a" => { "b" => { "c" => "3" } } }, @nested_strings.deep_transform_values { |value| value.to_s })
+    assert_equal({ "a" => [ { "b" => "2" }, { "c" => "3" }, "4" ] }, @string_array_of_hashes.deep_transform_values { |value| value.to_s })
+  end
+
+  def test_deep_transform_values_not_mutates
+    transformed_hash = @nested_mixed.deep_dup
+    transformed_hash.deep_transform_values { |value| value.to_s }
+    assert_equal @nested_mixed, transformed_hash
+  end
+
+  def test_deep_transform_values!
+    assert_equal({ "a" => "1", "b" => "2" }, @strings.deep_transform_values! { |value| value.to_s })
+    assert_equal({ "a" => { "b" => { "c" => "3" } } }, @nested_strings.deep_transform_values! { |value| value.to_s })
+    assert_equal({ "a" => [ { "b" => "2" }, { "c" => "3" }, "4" ] }, @string_array_of_hashes.deep_transform_values! { |value| value.to_s })
+  end
+
+  def test_deep_transform_values_with_bang_mutates
+    transformed_hash = @nested_mixed.deep_dup
+    transformed_hash.deep_transform_values! { |value| value.to_s }
+    assert_equal({ "a" => { b: { "c" => "3" } } }, transformed_hash)
+    assert_equal({ "a" => { b: { "c" => 3 } } }, @nested_mixed)
+  end
+
   def test_symbolize_keys
     assert_equal @symbols, @symbols.symbolize_keys
     assert_equal @symbols, @strings.symbolize_keys
@@ -339,30 +337,16 @@ class HashExtTest < ActiveSupport::TestCase
     assert_equal expected, merged
   end
 
-  def test_slice
-    original = { a: "x", b: "y", c: 10 }
-    expected = { a: "x", b: "y" }
-
-    # Should return a new hash with only the given keys.
-    assert_equal expected, original.slice(:a, :b)
-    assert_not_equal expected, original
-  end
-
   def test_slice_inplace
     original = { a: "x", b: "y", c: 10 }
-    expected = { c: 10 }
+    expected_return = { c: 10 }
+    expected_original = { a: "x", b: "y" }
+
+    # Should return a hash containing the removed key/value pairs.
+    assert_equal expected_return, original.slice!(:a, :b)
 
     # Should replace the hash with only the given keys.
-    assert_equal expected, original.slice!(:a, :b)
-  end
-
-  def test_slice_with_an_array_key
-    original = { :a => "x", :b => "y", :c => 10, [:a, :b] => "an array key" }
-    expected = { [:a, :b] => "an array key", :c => 10 }
-
-    # Should return a new hash with only the given keys when given an array key.
-    assert_equal expected, original.slice([:a, :b], :c)
-    assert_not_equal expected, original
+    assert_equal expected_original, original
   end
 
   def test_slice_inplace_with_an_array_key
@@ -373,14 +357,6 @@ class HashExtTest < ActiveSupport::TestCase
     assert_equal expected, original.slice!([:a, :b], :c)
   end
 
-  def test_slice_with_splatted_keys
-    original = { :a => "x", :b => "y", :c => 10, [:a, :b] => "an array key" }
-    expected = { a: "x", b: "y" }
-
-    # Should grab each of the splatted keys.
-    assert_equal expected, original.slice(*[:a, :b])
-  end
-
   def test_slice_bang_does_not_override_default
     hash = Hash.new(0)
     hash.update(a: 1, b: 2)
@@ -446,7 +422,7 @@ class HashExtTest < ActiveSupport::TestCase
     original.freeze
     assert_nothing_raised { original.except(:a) }
 
-    assert_raise(frozen_error_class) { original.except!(:a) }
+    assert_raise(FrozenError) { original.except!(:a) }
   end
 
   def test_except_does_not_delete_values_in_original
@@ -456,38 +432,10 @@ class HashExtTest < ActiveSupport::TestCase
     end
   end
 
-  def test_compact
-    hash_contain_nil_value = @symbols.merge(z: nil)
-    hash_with_only_nil_values = { a: nil, b: nil }
-
-    h = hash_contain_nil_value.dup
-    assert_equal(@symbols, h.compact)
-    assert_equal(hash_contain_nil_value, h)
-
-    h = hash_with_only_nil_values.dup
-    assert_equal({}, h.compact)
-    assert_equal(hash_with_only_nil_values, h)
-
-    h = @symbols.dup
-    assert_equal(@symbols, h.compact)
-    assert_equal(@symbols, h)
-  end
-
-  def test_compact!
-    hash_contain_nil_value = @symbols.merge(z: nil)
-    hash_with_only_nil_values = { a: nil, b: nil }
-
-    h = hash_contain_nil_value.dup
-    assert_equal(@symbols, h.compact!)
-    assert_equal(@symbols, h)
-
-    h = hash_with_only_nil_values.dup
-    assert_equal({}, h.compact!)
-    assert_equal({}, h)
-
-    h = @symbols.dup
-    assert_nil(h.compact!)
-    assert_equal(@symbols, h)
+  def test_requiring_compact_is_deprecated
+    assert_deprecated do
+      require "active_support/core_ext/hash/compact"
+    end
   end
 end
 
