@@ -8,14 +8,14 @@ class IntegerExtTest < ActiveSupport::TestCase
 
   def test_multiple_of
     [ -7, 0, 7, 14 ].each { |i| assert i.multiple_of?(7) }
-    [ -7, 7, 14 ].each { |i| assert ! i.multiple_of?(6) }
+    [ -7, 7, 14 ].each { |i| assert_not i.multiple_of?(6) }
 
     # test the 0 edge case
     assert 0.multiple_of?(0)
-    assert !5.multiple_of?(0)
+    assert_not 5.multiple_of?(0)
 
     # test with a prime
-    [2, 3, 5, 7].each { |i| assert !PRIME.multiple_of?(i) }
+    [2, 3, 5, 7].each { |i| assert_not PRIME.multiple_of?(i) }
   end
 
   def test_ordinalize
