@@ -17,7 +17,7 @@ class NameErrorTest < ActiveSupport::TestCase
     exc = assert_raise NameError do
       some_method_that_does_not_exist
     end
-    assert !exc.missing_name?(:Foo)
+    assert_not exc.missing_name?(:Foo)
     assert_nil exc.missing_name
   end
 end
