@@ -3,7 +3,6 @@
 require "abstract_unit"
 require "active_support/time"
 require "time_zone_test_helpers"
-require "active_support/core_ext/string/strip"
 require "yaml"
 
 class TimeWithZoneTest < ActiveSupport::TestCase
@@ -163,7 +162,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_to_yaml
-    yaml = <<-EOF.strip_heredoc
+    yaml = <<~EOF
       --- !ruby/object:ActiveSupport::TimeWithZone
       utc: 2000-01-01 00:00:00.000000000 Z
       zone: !ruby/object:ActiveSupport::TimeZone
@@ -175,7 +174,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_ruby_to_yaml
-    yaml = <<-EOF.strip_heredoc
+    yaml = <<~EOF
       ---
       twz: !ruby/object:ActiveSupport::TimeWithZone
         utc: 2000-01-01 00:00:00.000000000 Z
@@ -188,7 +187,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_yaml_load
-    yaml = <<-EOF.strip_heredoc
+    yaml = <<~EOF
       --- !ruby/object:ActiveSupport::TimeWithZone
       utc: 2000-01-01 00:00:00.000000000 Z
       zone: !ruby/object:ActiveSupport::TimeZone
@@ -200,7 +199,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_ruby_yaml_load
-    yaml = <<-EOF.strip_heredoc
+    yaml = <<~EOF
       ---
       twz: !ruby/object:ActiveSupport::TimeWithZone
         utc: 2000-01-01 00:00:00.000000000 Z
@@ -290,6 +289,20 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     end
   end
 
+  def test_before
+    twz = ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 0, 0), @time_zone)
+    assert_equal false, twz.before?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 11, 59, 59), @time_zone))
+    assert_equal false, twz.before?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 0, 0), @time_zone))
+    assert_equal true, twz.before?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 00, 1), @time_zone))
+  end
+
+  def test_after
+    twz = ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 0, 0), @time_zone)
+    assert_equal true, twz.after?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 11, 59, 59), @time_zone))
+    assert_equal false, twz.after?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 0, 0), @time_zone))
+    assert_equal false, twz.after?(ActiveSupport::TimeWithZone.new(Time.utc(2017, 3, 6, 12, 00, 1), @time_zone))
+  end
+
   def test_eql?
     assert_equal true, @twz.eql?(@twz.dup)
     assert_equal true, @twz.eql?(Time.utc(2000))
@@ -1092,7 +1105,7 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
   def test_use_zone_raises_on_invalid_timezone
     Time.zone = "Alaska"
     assert_raise ArgumentError do
-      Time.use_zone("No such timezone exists") {}
+      Time.use_zone("No such timezone exists") { }
     end
     assert_equal ActiveSupport::TimeZone["Alaska"], Time.zone
   end
