@@ -9,16 +9,16 @@ class ProxyWrappersTest < ActiveSupport::TestCase
 
   def test_deprecated_object_proxy_doesnt_wrap_falsy_objects
     proxy = ActiveSupport::Deprecation::DeprecatedObjectProxy.new(nil, "message")
-    assert !proxy
+    assert_not proxy
   end
 
   def test_deprecated_instance_variable_proxy_doesnt_wrap_falsy_objects
     proxy = ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(nil, :waffles)
-    assert !proxy
+    assert_not proxy
   end
 
   def test_deprecated_constant_proxy_doesnt_wrap_falsy_objects
     proxy = ActiveSupport::Deprecation::DeprecatedConstantProxy.new(Waffles, NewWaffles)
-    assert !proxy
+    assert_not proxy
   end
 end
