@@ -1,5 +1,8 @@
 # frozen_string_literal: true
 
+require "fileutils"
+require "tmpdir"
+
 module MultibyteTestHelpers
   class Downloader
     def self.download(from, to)
@@ -7,7 +10,7 @@ module MultibyteTestHelpers
         unless File.exist?(File.dirname(to))
           system "mkdir -p #{File.dirname(to)}"
         end
-        open(from) do |source|
+        URI.open(from) do |source|
           File.open(to, "w") do |target|
             source.each_line do |l|
               target.write l
@@ -23,9 +26,9 @@ module MultibyteTestHelpers
   CACHE_DIR = "#{Dir.tmpdir}/cache/unicode_conformance/#{ActiveSupport::Multibyte::Unicode::UNICODE_VERSION}"
   FileUtils.mkdir_p(CACHE_DIR)
 
-  UNICODE_STRING = "こにちわ".freeze
-  ASCII_STRING = "ohayo".freeze
-  BYTE_STRING = "\270\236\010\210\245".dup.force_encoding("ASCII-8BIT").freeze
+  UNICODE_STRING = "こにちわ"
+  ASCII_STRING = "ohayo"
+  BYTE_STRING = (+"\270\236\010\210\245").force_encoding("ASCII-8BIT").freeze
 
   def chars(str)
     ActiveSupport::Multibyte::Chars.new(str)
