@@ -20,12 +20,47 @@ module Notifications
     end
 
   private
-
     def event(*args)
       ActiveSupport::Notifications::Event.new(*args)
     end
   end
 
+  class SubscribeEventObjectsTest < TestCase
+    def test_subscribe_events
+      events = []
+      @notifier.subscribe do |event|
+        events << event
+      end
+
+      ActiveSupport::Notifications.instrument("foo")
+      event = events.first
+      assert event, "should have an event"
+      assert_operator event.allocations, :>, 0
+      assert_operator event.cpu_time, :>, 0
+      assert_operator event.idle_time, :>, 0
+      assert_operator event.duration, :>, 0
+    end
+
+    def test_subscribe_via_top_level_api
+      old_notifier = ActiveSupport::Notifications.notifier
+      ActiveSupport::Notifications.notifier = ActiveSupport::Notifications::Fanout.new
+
+      event = nil
+      ActiveSupport::Notifications.subscribe("foo") do |e|
+        event = e
+      end
+
+      ActiveSupport::Notifications.instrument("foo") do
+        100.times { Object.new } # allocate at least 100 objects
+      end
+
+      assert event
+      assert_operator event.allocations, :>=, 100
+    ensure
+      ActiveSupport::Notifications.notifier = old_notifier
+    end
+  end
+
   class SubscribedTest < TestCase
     def test_subscribed
       name     = "foo"
@@ -45,7 +80,7 @@ module Notifications
       assert_equal expected, events
     end
 
-    def test_subsribing_to_instrumentation_while_inside_it
+    def test_subscribing_to_instrumentation_while_inside_it
       # the repro requires that there are no evented subscribers for the "foo" event,
       # so we have to duplicate some of the setup code
       old_notifier = ActiveSupport::Notifications.notifier
@@ -54,7 +89,7 @@ module Notifications
       ActiveSupport::Notifications.subscribe("foo", TestSubscriber.new)
 
       ActiveSupport::Notifications.instrument("foo") do
-        ActiveSupport::Notifications.subscribe("foo") {}
+        ActiveSupport::Notifications.subscribe("foo") { }
       end
     ensure
       ActiveSupport::Notifications.notifier = old_notifier
@@ -92,6 +127,25 @@ module Notifications
       assert_equal [["named.subscription", :foo], ["named.subscription", :foo]], @events
     end
 
+    def test_unsubscribing_by_name_leaves_regexp_matched_subscriptions
+      @matched_events = []
+      @notifier.subscribe(/subscription/) { |*args| @matched_events << event(*args) }
+      @notifier.publish("named.subscription", :before)
+      @notifier.wait
+      [@events, @named_events, @matched_events].each do |collector|
+        assert_includes(collector, ["named.subscription", :before])
+      end
+      @notifier.unsubscribe("named.subscription")
+      @notifier.publish("named.subscription", :after)
+      @notifier.publish("other.subscription", :after)
+      @notifier.wait
+      assert_includes(@events, ["named.subscription", :after])
+      assert_includes(@events, ["other.subscription", :after])
+      assert_includes(@matched_events, ["other.subscription", :after])
+      assert_not_includes(@matched_events, ["named.subscription", :after])
+      assert_not_includes(@named_events, ["named.subscription", :after])
+    end
+
   private
     def event(*args)
       args
@@ -255,24 +309,32 @@ module Notifications
       assert_in_delta 10.0, event.duration, 0.00001
     end
 
+    def test_event_cpu_time_does_not_raise_error_when_start_or_finished_not_called
+      time = Time.now
+      event = event(:foo, time, time + 0.01, random_id, {})
+
+      assert_equal 0, event.cpu_time
+    end
+
+
     def test_events_consumes_information_given_as_payload
-      event = event(:foo, Time.now, Time.now + 1, random_id, payload: :bar)
+      event = event(:foo, Concurrent.monotonic_time, Concurrent.monotonic_time + 1, random_id, payload: :bar)
       assert_equal Hash[payload: :bar], event.payload
     end
 
     def test_event_is_parent_based_on_children
-      time = Time.utc(2009, 01, 01, 0, 0, 1)
+      time = Concurrent.monotonic_time
 
-      parent    = event(:foo, Time.utc(2009), Time.utc(2009) + 100, random_id, {})
+      parent    = event(:foo, Concurrent.monotonic_time, Concurrent.monotonic_time + 100, random_id, {})
       child     = event(:foo, time, time + 10, random_id, {})
       not_child = event(:foo, time, time + 100, random_id, {})
 
       parent.children << child
 
       assert parent.parent_of?(child)
-      assert !child.parent_of?(parent)
-      assert !parent.parent_of?(not_child)
-      assert !not_child.parent_of?(parent)
+      assert_not child.parent_of?(parent)
+      assert_not parent.parent_of?(not_child)
+      assert_not not_child.parent_of?(parent)
     end
 
     private
