@@ -5,7 +5,7 @@ require "qunit/selenium/test_runner"
 if ARGV[1]
   driver = ::Selenium::WebDriver.for(:remote, url: ARGV[1], desired_capabilities: :chrome)
 else
-  require "chromedriver-helper"
+  require "webdrivers"
 
   driver_options = Selenium::WebDriver::Chrome::Options.new
   driver_options.add_argument("--headless")
