@@ -9,10 +9,10 @@ commands = [
   'mysql -e "grant all privileges on activerecord_unittest.* to rails@localhost;"',
   'mysql -e "grant all privileges on activerecord_unittest2.* to rails@localhost;"',
   'mysql -e "grant all privileges on inexistent_activerecord_unittest.* to rails@localhost;"',
-  'mysql -e "create database activerecord_unittest;"',
-  'mysql -e "create database activerecord_unittest2;"',
-  'psql  -c "create database activerecord_unittest;" -U postgres',
-  'psql  -c "create database activerecord_unittest2;" -U postgres'
+  'mysql -e "create database activerecord_unittest default character set utf8mb4;"',
+  'mysql -e "create database activerecord_unittest2 default character set utf8mb4;"',
+  'psql  -c "create database -E UTF8 -T template0 activerecord_unittest;" -U postgres',
+  'psql  -c "create database -E UTF8 -T template0 activerecord_unittest2;" -U postgres'
 ]
 
 commands.each do |command|
@@ -20,20 +20,6 @@ commands.each do |command|
 end
 
 class Build
-  MAP = {
-    "railties" => "railties",
-    "ap"       => "actionpack",
-    "am"       => "actionmailer",
-    "amo"      => "activemodel",
-    "as"       => "activesupport",
-    "ar"       => "activerecord",
-    "av"       => "actionview",
-    "aj"       => "activejob",
-    "ac"       => "actioncable",
-    "ast"      => "activestorage",
-    "guides"   => "guides"
-  }
-
   attr_reader :component, :options
 
   def initialize(component, options = {})
@@ -114,7 +100,7 @@ class Build
   end
 
   def gem
-    MAP[component.split(":").first]
+    component.split(":").first
   end
   alias :dir :gem
 
@@ -149,7 +135,7 @@ class Build
   end
 end
 
-if ENV["GEM"] == "aj:integration"
+if ENV["GEM"] == "activejob:integration"
   ENV["QC_DATABASE_URL"]  = "postgres://postgres@localhost/active_jobs_qc_int_test"
   ENV["QUE_DATABASE_URL"] = "postgres://postgres@localhost/active_jobs_que_int_test"
 end
@@ -159,14 +145,16 @@ results = {}
 ENV["GEM"].split(",").each do |gem|
   [false, true].each do |isolated|
     next if ENV["TRAVIS_PULL_REQUEST"] && ENV["TRAVIS_PULL_REQUEST"] != "false" && isolated
-    next if RUBY_VERSION < "2.5" && isolated
+    next if RUBY_VERSION < "2.6" && isolated
     next if gem == "railties" && isolated
-    next if gem == "ac" && isolated
-    next if gem == "ac:integration" && isolated
-    next if gem == "aj:integration" && isolated
+    next if gem == "actioncable" && isolated
+    next if gem == "actioncable:integration" && isolated
+    next if gem == "activejob:integration" && isolated
     next if gem == "guides" && isolated
-    next if gem == "av:ujs" && isolated
-    next if gem == "ast" && isolated
+    next if gem == "actionview:ujs" && isolated
+    next if gem == "activestorage" && isolated
+    next if gem == "actionmailbox" && isolated
+    next if gem == "actiontext" && isolated
 
     build = Build.new(gem, isolated: isolated)
     results[build.key] = build.run!
