@@ -1,11 +1,6 @@
 # frozen_string_literal: true
 
-begin
-  require "bundler/inline"
-rescue LoadError => e
-  $stderr.puts "Bundler version 1.10 or later is required. Please update your Bundler"
-  raise e
-end
+require "bundler/inline"
 
 gemfile(true) do
   source "https://rubygems.org"
@@ -13,7 +8,7 @@ gemfile(true) do
   git_source(:github) { |repo| "https://github.com/#{repo}.git" }
 
   # Activate the gem you are reporting the issue against.
-  gem "rails", "5.2.0"
+  gem "rails", "6.0.0"
 end
 
 require "rack/test"
@@ -21,6 +16,7 @@ require "action_controller/railtie"
 
 class TestApp < Rails::Application
   config.root = __dir__
+  config.hosts << "example.org"
   config.session_store :cookie_store, key: "cookie_store_key"
   secrets.secret_key_base = "secret_key_base"
 
@@ -42,9 +38,6 @@ end
 
 require "minitest/autorun"
 
-# Ensure backward compatibility with Minitest 4
-Minitest::Test = MiniTest::Unit::TestCase unless defined?(Minitest::Test)
-
 class BugTest < Minitest::Test
   include Rack::Test::Methods
 
