@@ -8,6 +8,8 @@ class CodeStatistics #:nodoc:
                 "Helper tests",
                 "Model tests",
                 "Mailer tests",
+                "Mailbox tests",
+                "Channel tests",
                 "Job tests",
                 "Integration tests",
                 "System tests"]
@@ -46,7 +48,7 @@ class CodeStatistics #:nodoc:
 
         if File.directory?(path) && (/^\./ !~ file_name)
           stats.add(calculate_directory_statistics(path, pattern))
-        elsif file_name =~ pattern
+        elsif file_name&.match?(pattern)
           stats.add_by_file_path(path)
         end
       end
@@ -95,8 +97,8 @@ class CodeStatistics #:nodoc:
     end
 
     def print_line(name, statistics)
-      m_over_c   = (statistics.methods / statistics.classes) rescue m_over_c = 0
-      loc_over_m = (statistics.code_lines / statistics.methods) - 2 rescue loc_over_m = 0
+      m_over_c   = (statistics.methods / statistics.classes) rescue 0
+      loc_over_m = (statistics.code_lines / statistics.methods) - 2 rescue 0
 
       print "| #{name.ljust(20)} "
       HEADERS.each_key do |k|
