@@ -15,7 +15,7 @@ class Rails::Command::CredentialsCommandTest < ActiveSupport::TestCase
   test "edit without editor gives hint" do
     run_edit_command(editor: "").tap do |output|
       assert_match "No $EDITOR to open file in", output
-      assert_match "bin/rails credentials:edit", output
+      assert_match "rails credentials:edit", output
     end
   end
 
@@ -49,25 +49,50 @@ class Rails::Command::CredentialsCommandTest < ActiveSupport::TestCase
       FileUtils.rm("config/master.key")
 
       switch_env("RAILS_MASTER_KEY", key) do
-        run_edit_command
+        assert_match(/access_key_id: 123/, run_edit_command)
         assert_not File.exist?("config/master.key")
       end
     end
   end
 
-  test "edit command does not raise when an initializer tries to acces non-existent credentials" do
+  test "edit command modifies file specified by environment option" do
+    assert_match(/access_key_id: 123/, run_edit_command(environment: "production"))
+    Dir.chdir(app_path) do
+      assert File.exist?("config/credentials/production.key")
+      assert File.exist?("config/credentials/production.yml.enc")
+    end
+  end
+
+  test "edit command properly expands environment option" do
+    assert_match(/access_key_id: 123/, run_edit_command(environment: "prod"))
+    Dir.chdir(app_path) do
+      assert File.exist?("config/credentials/production.key")
+      assert File.exist?("config/credentials/production.yml.enc")
+    end
+  end
+
+  test "edit command does not raise when an initializer tries to access non-existent credentials" do
     app_file "config/initializers/raise_when_loaded.rb", <<-RUBY
       Rails.application.credentials.missing_key!
     RUBY
 
-    assert_match(/access_key_id: 123/, run_edit_command)
+    assert_match(/access_key_id: 123/, run_edit_command(environment: "qa"))
+  end
+
+  test "edit command generates template file when the file does not exist" do
+    FileUtils.rm("#{app_path}/config/credentials.yml.enc")
+    run_edit_command
+
+    output = run_show_command
+    assert_match(/access_key_id: 123/, output)
+    assert_match(/secret_key_base/, output)
   end
 
   test "show credentials" do
     assert_match(/access_key_id: 123/, run_show_command)
   end
 
-  test "show command raise error when require_master_key is specified and key does not exist" do
+  test "show command raises error when require_master_key is specified and key does not exist" do
     remove_file "config/master.key"
     add_to_config "config.require_master_key = true"
 
@@ -78,17 +103,33 @@ class Rails::Command::CredentialsCommandTest < ActiveSupport::TestCase
     remove_file "config/master.key"
     add_to_config "config.require_master_key = false"
 
-    assert_match(/Missing master key to decrypt credentials/, run_show_command)
+    assert_match(/Missing 'config\/master\.key' to decrypt credentials/, run_show_command)
+  end
+
+  test "show command displays content specified by environment option" do
+    run_edit_command(environment: "production")
+
+    assert_match(/access_key_id: 123/, run_show_command(environment: "production"))
+  end
+
+  test "show command properly expands environment option" do
+    run_edit_command(environment: "production")
+
+    output = run_show_command(environment: "prod")
+    assert_match(/access_key_id: 123/, output)
+    assert_no_match(/secret_key_base/, output)
   end
 
   private
-    def run_edit_command(editor: "cat")
+    def run_edit_command(editor: "cat", environment: nil, **options)
       switch_env("EDITOR", editor) do
-        rails "credentials:edit"
+        args = environment ? ["--environment", environment] : []
+        rails "credentials:edit", args, **options
       end
     end
 
-    def run_show_command(**options)
-      rails "credentials:show", **options
+    def run_show_command(environment: nil, **options)
+      args = environment ? ["--environment", environment] : []
+      rails "credentials:show", args, **options
     end
 end
