@@ -125,7 +125,7 @@ class ActionsTest < Rails::Generators::TestCase
   def test_gem_works_even_if_frozen_string_is_passed_as_argument
     run_generator
 
-    action :gem, "frozen_gem".freeze, "1.0.0".freeze
+    action :gem, -"frozen_gem", -"1.0.0"
 
     assert_file "Gemfile", /^gem 'frozen_gem', '1.0.0'$/
   end
@@ -144,6 +144,44 @@ class ActionsTest < Rails::Generators::TestCase
     assert_file "Gemfile", /\ngroup :development, :test do\n  gem 'rspec-rails'\nend\n\ngroup :test do\n  gem 'fakeweb'\nend/
   end
 
+  def test_github_should_create_an_indented_block
+    run_generator
+
+    action :github, "user/repo" do
+      gem "foo"
+      gem "bar"
+      gem "baz"
+    end
+
+    assert_file "Gemfile", /\ngithub 'user\/repo' do\n  gem 'foo'\n  gem 'bar'\n  gem 'baz'\nend/
+  end
+
+  def test_github_should_create_an_indented_block_with_options
+    run_generator
+
+    action :github, "user/repo", a: "correct", other: true do
+      gem "foo"
+      gem "bar"
+      gem "baz"
+    end
+
+    assert_file "Gemfile", /\ngithub 'user\/repo', a: 'correct', other: true do\n  gem 'foo'\n  gem 'bar'\n  gem 'baz'\nend/
+  end
+
+  def test_github_should_create_an_indented_block_within_a_group
+    run_generator
+
+    action :gem_group, :magic do
+      github "user/repo", a: "correct", other: true do
+        gem "foo"
+        gem "bar"
+        gem "baz"
+      end
+    end
+
+    assert_file "Gemfile", /\ngroup :magic do\n  github 'user\/repo', a: 'correct', other: true do\n    gem 'foo'\n    gem 'bar'\n    gem 'baz'\n  end\nend\n/
+  end
+
   def test_environment_should_include_data_in_environment_initializer_block
     run_generator
     autoload_paths = 'config.autoload_paths += %w["#{Rails.root}/app/extras"]'
@@ -162,7 +200,7 @@ class ActionsTest < Rails::Generators::TestCase
     run_generator
 
     action :environment do
-      _ = "# This wont be added"# assignment to silence parse-time warning "unused literal ignored"
+      _ = "# This wont be added" # assignment to silence parse-time warning "unused literal ignored"
       "# This will be added"
     end
 
@@ -265,12 +303,30 @@ class ActionsTest < Rails::Generators::TestCase
   end
 
   def test_generate_should_run_script_generate_with_argument_and_options
-    assert_called_with(generator, :run_ruby_script, ["bin/rails generate model MyModel", verbose: false]) do
-      action :generate, "model", "MyModel"
+    run_generator
+    action :generate, "model", "MyModel"
+    assert_file "app/models/my_model.rb", /MyModel/
+  end
+
+  def test_generate_aborts_when_subprocess_fails_if_requested
+    run_generator
+    content = capture(:stderr) do
+      assert_raises SystemExit do
+        action :generate, "model", "MyModel:ADsad", abort_on_failure: true
+        action :generate, "model", "MyModel"
+      end
+    end
+    assert_match(/wrong constant name MyModel:aDsad/, content)
+    assert_no_file "app/models/my_model.rb"
+  end
+
+  def test_generate_should_run_command_without_env
+    assert_called_with(generator, :run, ["rails generate model MyModel name:string", verbose: false]) do
+      action :generate, "model", "MyModel", "name:string"
     end
   end
 
-  def test_rails_should_run_rake_command_with_default_env
+  def test_rake_should_run_rake_command_with_default_env
     assert_called_with(generator, :run, ["rake log:clear RAILS_ENV=development", verbose: false]) do
       with_rails_env nil do
         action :rake, "log:clear"
@@ -278,13 +334,13 @@ class ActionsTest < Rails::Generators::TestCase
     end
   end
 
-  def test_rails_with_env_option_should_run_rake_command_in_env
+  def test_rake_with_env_option_should_run_rake_command_in_env
     assert_called_with(generator, :run, ["rake log:clear RAILS_ENV=production", verbose: false]) do
       action :rake, "log:clear", env: "production"
     end
   end
 
-  test "rails command with RAILS_ENV variable should run rake command in env" do
+  test "rake with RAILS_ENV variable should run rake command in env" do
     assert_called_with(generator, :run, ["rake log:clear RAILS_ENV=production", verbose: false]) do
       with_rails_env "production" do
         action :rake, "log:clear"
@@ -300,7 +356,7 @@ class ActionsTest < Rails::Generators::TestCase
     end
   end
 
-  test "rails command with sudo option should run rake command with sudo" do
+  test "rake with sudo option should run rake command with sudo" do
     assert_called_with(generator, :run, ["sudo rake log:clear RAILS_ENV=development", verbose: false]) do
       with_rails_env nil do
         action :rake, "log:clear", sudo: true
@@ -362,15 +418,6 @@ class ActionsTest < Rails::Generators::TestCase
     end
   end
 
-  def test_capify_should_run_the_capify_command
-    content = capture(:stderr) do
-      assert_called_with(generator, :run, ["capify .", verbose: false]) do
-        action :capify!
-      end
-    end
-    assert_match(/DEPRECATION WARNING: `capify!` is deprecated/, content)
-  end
-
   def test_route_should_add_data_to_the_routes_block_in_config_routes
     run_generator
     route_command = "route '/login', controller: 'sessions', action: 'new'"
@@ -403,13 +450,13 @@ class ActionsTest < Rails::Generators::TestCase
     content.gsub!(/^  \#.*\n/, "")
     content.gsub!(/^\n/, "")
 
-    File.open(route_path, "wb") { |file| file.write(content) }
+    File.write(route_path, content)
 
     routes = <<-F
 Rails.application.routes.draw do
   root 'welcome#index'
 end
-F
+    F
 
     assert_file "config/routes.rb", routes
 
@@ -420,7 +467,7 @@ Rails.application.routes.draw do
   resources :product_lines
   root 'welcome#index'
 end
-F
+    F
     assert_file "config/routes.rb", routes
   end
 
@@ -458,7 +505,6 @@ F
   end
 
   private
-
     def action(*args, &block)
       capture(:stdout) { generator.send(*args, &block) }
     end
