@@ -39,13 +39,11 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
 
   def test_invokes_assets
     run_generator
-    assert_file "app/assets/javascripts/account.js"
     assert_file "app/assets/stylesheets/account.css"
   end
 
   def test_does_not_invoke_assets_if_required
     run_generator ["account", "--skip-assets"]
-    assert_no_file "app/assets/javascripts/account.js"
     assert_no_file "app/assets/stylesheets/account.css"
   end
 
@@ -116,4 +114,23 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
       assert_no_match(/namespace :admin/, routes)
     end
   end
+
+  def test_controller_suffix_is_not_duplicated
+    run_generator ["account_controller"]
+
+    assert_no_file "app/controllers/account_controller_controller.rb"
+    assert_file "app/controllers/account_controller.rb"
+
+    assert_no_file "app/views/account_controller/"
+    assert_file "app/views/account/"
+
+    assert_no_file "test/controllers/account_controller_controller_test.rb"
+    assert_file "test/controllers/account_controller_test.rb"
+
+    assert_no_file "app/helpers/account_controller_helper.rb"
+    assert_file "app/helpers/account_helper.rb"
+
+    assert_no_file "app/assets/stylesheets/account_controller.css"
+    assert_file "app/assets/stylesheets/account.css"
+  end
 end
