@@ -2,12 +2,24 @@
 
 require "generators/generators_test_helper"
 require "rails/generators/rails/model/model_generator"
-require "active_support/core_ext/string/strip"
 
 class ModelGeneratorTest < Rails::Generators::TestCase
   include GeneratorsTestHelper
   arguments %w(Account name:string age:integer)
 
+  def setup
+    super
+    Rails::Generators::ModelHelpers.skip_warn = false
+    @old_belongs_to_required_by_default = Rails.application.config.active_record.belongs_to_required_by_default
+
+    Rails.application.config.active_record.belongs_to_required_by_default = true
+  end
+
+
+  def teardown
+    Rails.application.config.active_record.belongs_to_required_by_default = @old_belongs_to_required_by_default
+  end
+
   def test_help_shows_invoked_generators_options
     content = run_generator ["--help"]
     assert_match(/ActiveRecord options:/, content)
@@ -38,12 +50,24 @@ class ModelGeneratorTest < Rails::Generators::TestCase
   end
 
   def test_plural_names_are_singularized
-    content = run_generator ["accounts".freeze]
+    content = run_generator ["accounts"]
     assert_file "app/models/account.rb", /class Account < ApplicationRecord/
     assert_file "test/models/account_test.rb", /class AccountTest/
     assert_match(/\[WARNING\] The model name 'accounts' was recognized as a plural, using the singular 'account' instead\. Override with --force-plural or setup custom inflection rules for this noun before running the generator\./, content)
   end
 
+  def test_unknown_inflection_rule_are_warned
+    content = run_generator ["porsche"]
+    assert_match("[WARNING] Rails cannot recover singular form from its plural form 'porsches'.\nPlease setup custom inflection rules for this noun before running the generator in config/initializers/inflections.rb.", content)
+    assert_file "app/models/porsche.rb", /class Porsche < ApplicationRecord/
+
+    uncountable_content = run_generator ["sheep"]
+    assert_no_match("[WARNING] Rails cannot recover singular form from its plural form", uncountable_content)
+
+    regular_content = run_generator ["account"]
+    assert_no_match("[WARNING] Rails cannot recover singular form from its plural form", regular_content)
+  end
+
   def test_model_with_underscored_parent_option
     run_generator ["account", "--parent", "admin/account"]
     assert_file "app/models/account.rb", /class Account < Admin::Account/
@@ -88,7 +112,7 @@ class ModelGeneratorTest < Rails::Generators::TestCase
     ActiveRecord::Base.pluralize_table_names = true
   end
 
-  def test_migration_with_namespaces_in_model_name_without_plurization
+  def test_migration_with_namespaces_in_model_name_without_pluralization
     ActiveRecord::Base.pluralize_table_names = false
     run_generator ["Gallery::Image"]
     assert_migration "db/migrate/create_gallery_image", /class CreateGalleryImage < ActiveRecord::Migration\[[0-9.]+\]/
@@ -376,41 +400,43 @@ class ModelGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_required_belongs_to_adds_required_association
-    run_generator ["account", "supplier:references{required}"]
-
-    expected_file = <<-FILE.strip_heredoc
-    class Account < ApplicationRecord
-      belongs_to :supplier, required: true
+  def test_database_puts_migrations_in_configured_folder
+    with_secondary_database_configuration do
+      run_generator ["account", "--database=secondary"]
+      assert_migration "db/secondary_migrate/create_accounts.rb" do |content|
+        assert_method :change, content do |change|
+          assert_match(/create_table :accounts/, change)
+        end
+      end
     end
+  end
+
+  def test_database_puts_migrations_in_configured_folder_with_aliases
+    with_secondary_database_configuration do
+      run_generator ["account", "--db=secondary"]
+      assert_migration "db/secondary_migrate/create_accounts.rb" do |content|
+        assert_method :change, content do |change|
+          assert_match(/create_table :accounts/, change)
+        end
+      end
+    end
+  end
+
+  def test_polymorphic_belongs_to_generates_correct_model
+    run_generator ["account", "supplier:references{polymorphic}"]
+
+    expected_file = <<~FILE
+      class Account < ApplicationRecord
+        belongs_to :supplier, polymorphic: true
+      end
     FILE
     assert_file "app/models/account.rb", expected_file
   end
 
-  def test_required_polymorphic_belongs_to_generages_correct_model
-    run_generator ["account", "supplier:references{required,polymorphic}"]
-
-    expected_file = <<-FILE.strip_heredoc
-    class Account < ApplicationRecord
-      belongs_to :supplier, polymorphic: true, required: true
+  def test_passing_required_to_model_generator_is_deprecated
+    assert_deprecated do
+      run_generator ["account", "supplier:references{required}"]
     end
-    FILE
-    assert_file "app/models/account.rb", expected_file
-  end
-
-  def test_required_and_polymorphic_are_order_independent
-    run_generator ["account", "supplier:references{polymorphic.required}"]
-
-    expected_file = <<-FILE.strip_heredoc
-    class Account < ApplicationRecord
-      belongs_to :supplier, polymorphic: true, required: true
-    end
-    FILE
-    assert_file "app/models/account.rb", expected_file
-  end
-
-  def test_required_adds_null_false_to_column
-    run_generator ["account", "supplier:references{required}"]
 
     assert_migration "db/migrate/create_accounts.rb" do |m|
       assert_method :change, m do |up|
@@ -419,6 +445,38 @@ class ModelGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_null_false_is_added_for_references_by_default
+    run_generator ["account", "user:references"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_match(/t\.references :user,.*\snull: false/, up)
+      end
+    end
+  end
+
+  def test_null_false_is_added_for_belongs_to_by_default
+    run_generator ["account", "user:belongs_to"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_match(/t\.belongs_to :user,.*\snull: false/, up)
+      end
+    end
+  end
+
+  def test_null_false_is_not_added_when_belongs_to_required_by_default_global_config_is_false
+    Rails.application.config.active_record.belongs_to_required_by_default = false
+
+    run_generator ["account", "user:belongs_to"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_match(/t\.belongs_to :user/, up)
+      end
+    end
+  end
+
   def test_foreign_key_is_not_added_for_non_references
     run_generator ["account", "supplier:string"]
 
@@ -452,15 +510,52 @@ class ModelGeneratorTest < Rails::Generators::TestCase
 
   def test_token_option_adds_has_secure_token
     run_generator ["user", "token:token", "auth_token:token"]
-    expected_file = <<-FILE.strip_heredoc
-    class User < ApplicationRecord
-      has_secure_token
-      has_secure_token :auth_token
-    end
+    expected_file = <<~FILE
+      class User < ApplicationRecord
+        has_secure_token
+        has_secure_token :auth_token
+      end
     FILE
     assert_file "app/models/user.rb", expected_file
   end
 
+  def test_model_with_rich_text_attribute_adds_has_rich_text
+    run_generator ["message", "content:rich_text"]
+    expected_file = <<~FILE
+      class Message < ApplicationRecord
+        has_rich_text :content
+      end
+    FILE
+    assert_file "app/models/message.rb", expected_file
+  end
+
+  def test_model_with_attachment_attribute_adds_has_one_attached
+    run_generator ["message", "video:attachment"]
+    expected_file = <<~FILE
+      class Message < ApplicationRecord
+        has_one_attached :video
+      end
+    FILE
+    assert_file "app/models/message.rb", expected_file
+  end
+
+  def test_model_with_attachments_attribute_adds_has_many_attached
+    run_generator ["message", "photos:attachments"]
+    expected_file = <<~FILE
+      class Message < ApplicationRecord
+        has_many_attached :photos
+      end
+    FILE
+    assert_file "app/models/message.rb", expected_file
+  end
+
+  def test_skip_virtual_fields_in_fixtures
+    run_generator ["message", "content:rich_text", "video:attachment", "photos:attachments"]
+
+    assert_generated_fixture("test/fixtures/messages.yml",
+                             "one" => nil, "two" => nil)
+  end
+
   private
     def assert_generated_fixture(path, parsed_contents)
       fixture_file = File.new File.expand_path(path, destination_root)
