@@ -28,12 +28,13 @@ class GeneratorsTest < Rails::Generators::TestCase
     output = capture(:stdout) { Rails::Generators.invoke name }
     assert_match "Could not find generator '#{name}'", output
     assert_match "`rails generate --help`", output
+    assert_no_match "Maybe you meant", output
   end
 
   def test_generator_suggestions
     name = :migrationz
     output = capture(:stdout) { Rails::Generators.invoke name }
-    assert_match "Maybe you meant 'migration'", output
+    assert_match 'Maybe you meant "migration"?', output
   end
 
   def test_generator_suggestions_except_en_locale
@@ -43,18 +44,12 @@ class GeneratorsTest < Rails::Generators::TestCase
     I18n.default_locale = :ja
     name = :tas
     output = capture(:stdout) { Rails::Generators.invoke name }
-    assert_match "Maybe you meant 'task', 'job' or", output
+    assert_match 'Maybe you meant "task"?', output
   ensure
     I18n.available_locales = orig_available_locales
     I18n.default_locale = orig_default_locale
   end
 
-  def test_generator_multiple_suggestions
-    name = :tas
-    output = capture(:stdout) { Rails::Generators.invoke name }
-    assert_match "Maybe you meant 'task', 'job' or", output
-  end
-
   def test_help_when_a_generator_with_required_arguments_is_invoked_without_arguments
     output = capture(:stdout) { Rails::Generators.invoke :model, [] }
     assert_match(/Description:/, output)
